@@ -4,11 +4,11 @@
 /*                                                                          */
 /*                             A - A D A I N T                              */
 /*                                                                          */
-/*                            $Revision: 1.119 $
+/*                            $Revision: 1.145-3.14p  $
 /*                                                                          */
 /*                          C Implementation File                           */
 /*                                                                          */
-/*          Copyright (C) 1992-1999, Free Software Foundation, Inc.         */
+/*          Copyright (C) 1992-2001, Free Software Foundation, Inc.         */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -38,6 +38,10 @@
 /*  directly. This file contains all other routines.                        */
 
 #ifdef __vxworks
+/* No need to redefine exit here */
+#ifdef exit
+#undef exit
+#endif
 #include "vxWorks.h"
 
 #if defined (__mips_vxworks)
@@ -50,6 +54,7 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <fcntl.h>
 #include <time.h>
 #include <unistd.h>
@@ -62,6 +67,7 @@
 
 #if defined (_WIN32)
 #include <dir.h>
+#include <windows.h>
 #endif
 
 #include "a-adaint.h"
@@ -73,7 +79,6 @@
    no text translation is required, so these flags have no effect.
 */
 
-
 #if defined (__EMX__)
 #include <os2.h>
 #endif
@@ -204,7 +209,7 @@ Try_Lock (dir, file)
   return 1;
 }
 
-#elif defined (__EMX__) || defined (VMS) 
+#elif defined (__EMX__) || defined (VMS)
 
 /* More cases that do not use link() */
 /* identical code, to solve too long line problem ??? */
@@ -226,7 +231,6 @@ Try_Lock (dir, file)
   return 1;
 }
 
-
 #else
 
 /* Version using link(), more secure over NFS */
@@ -256,7 +260,7 @@ Try_Lock (dir, file)
 
   /* Count the references on the old one. If we have a count of two, then
    * the link did succeed. Remove the temporary file before returning */
-  stat (temp_file, &stat_result);
+  __gnat_stat (temp_file, &stat_result);
   unlink (temp_file);
   return stat_result.st_nlink == 2;
 }
@@ -331,15 +335,18 @@ Get_Pathsep_Char ()
 /* Return the current working directory */
 void Get_Current_Dir (char *dir, int *length)
 {
+#ifdef VMS
+   /* Force Unix style, which is what GNAT uses internally */
+   getcwd (dir, *length, 0);
+#else
    getcwd (dir, *length);
+#endif
 
    *length = strlen (dir);
 
-#ifndef VMS
    dir [*length] = DIR_SEPARATOR;
    ++(*length);
    dir [*length] = '\0';
-#endif
 }
 
 /* Return the suffix for object files */
@@ -407,7 +414,7 @@ open_read (path, fmode)
 #if defined(VMS)
   /* Optional arguments mbc,deq,fop increase read performance */
   fd = open (path, O_RDONLY | o_fmode, 0444,
-	     "mbc=16", "deq=64", "fop=tef");
+             "mbc=16", "deq=64", "fop=tef");
 #elif defined(__vxworks)
   fd = open (path, O_RDONLY | o_fmode, 0444);
 #else
@@ -417,7 +424,6 @@ open_read (path, fmode)
   return fd < 0 ? -1 : fd;
 }
 
-
 #if defined (__EMX__)
 #define PERM (S_IREAD | S_IWRITE)
 #else
@@ -437,7 +443,7 @@ open_rw (path, fmode)
 
 #if defined(VMS)
   fd = open (path, O_RDWR | o_fmode, PERM,
-	     "mbc=16", "deq=64", "fop=tef");
+             "mbc=16", "deq=64", "fop=tef");
 #else
   fd = open (path, O_RDWR | o_fmode, PERM);
 #endif
@@ -458,7 +464,7 @@ open_create (path, fmode)
 
 #if defined(VMS)
   fd = open (path, O_WRONLY | O_CREAT | O_TRUNC | o_fmode, PERM,
-	     "mbc=16", "deq=64", "fop=tef");
+             "mbc=16", "deq=64", "fop=tef");
 #else
   fd = open (path, O_WRONLY | O_CREAT | O_TRUNC | o_fmode, PERM);
 #endif
@@ -479,7 +485,7 @@ open_append (path, fmode)
 
 #if defined(VMS)
   fd = open (path, O_WRONLY | O_CREAT | O_APPEND | o_fmode, PERM,
-	     "mbc=16", "deq=64", "fop=tef");
+             "mbc=16", "deq=64", "fop=tef");
 #else
   fd = open (path, O_WRONLY | O_CREAT | O_APPEND | o_fmode, PERM);
 #endif
@@ -502,7 +508,7 @@ open_new (path, fmode)
 
 #if defined(VMS)
   fd = open (path, O_WRONLY | O_CREAT | O_EXCL | o_fmode, PERM,
-	     "mbc=16", "deq=64", "fop=tef");
+             "mbc=16", "deq=64", "fop=tef");
 #else
   fd = open (path, O_WRONLY | O_CREAT | O_EXCL | o_fmode, PERM);
 #endif
@@ -528,8 +534,8 @@ open_new_temp (path, fmode)
 
 #if defined(VMS)
   fd = open (path, O_WRONLY | O_CREAT | O_EXCL | o_fmode, PERM,
-	     "rfm=stmlf", "ctx=rec", "rat=none", "shr=del,get,put,upd",
-	     "mbc=16", "deq=64", "fop=tef");
+             "rfm=stmlf", "ctx=rec", "rat=none", "shr=del,get,put,upd",
+             "mbc=16", "deq=64", "fop=tef");
 #else
   fd = open (path, O_WRONLY | O_CREAT | O_EXCL | o_fmode, PERM);
 #endif
@@ -537,6 +543,18 @@ open_new_temp (path, fmode)
   return fd < 0 ? -1 : fd;
 }
 
+int
+__gnat_mkdir (char *dir_name)
+{
+#if defined (_WIN32)
+  return mkdir (dir_name);
+#elif defined (__vxworks)
+  return mkdir (dir_name);
+#else
+  return mkdir (dir_name, S_IRWXU | S_IRWXG | S_IRWXO);
+#endif
+}
+
 /* Return the number of bytes in the specified file. */
 
 long
@@ -553,6 +571,30 @@ file_length (fd)
   return (statbuf.st_size);
 }
 
+/* create a temporary filename and put it in string pointed to 
+   by tmp_filename */
+
+void
+__gnat_tmp_name (char *tmp_filename)
+{
+#ifdef __MINGW32__
+  {
+    char *pname;
+
+    /* tempnam try to create a temporary file in directory pointed to by TMP
+       environment variable, in c:\temp if TMP is not set, and in directory
+       specified by P_tmpdir in stdio.h if c:\temp does not exist. The
+       filename will be created with the prefix "gnat-" */
+
+    pname = (char *)tempnam ("c:\\temp", "gnat-");
+    strcpy (tmp_filename, pname);
+    free (pname);
+  }
+#else
+  tmpnam (tmp_filename);
+#endif
+}
+
 /* Read the next entry in a directory
    The returned string points somewhere in the buffer */
 
@@ -594,6 +636,40 @@ readdir_is_thread_safe ()
 #endif
 }
 
+#ifdef _WIN32
+/* 
+   win32_filetime returns the file modification timestamp using Win32 routines
+   which are immune against daylight saving time change. It is in fact not
+   possible to use fstat for this purpose as the DST modify the st_mtime field
+   of the stat structure.
+*/
+
+time_t
+win32_filetime (HANDLE h)
+{
+  BOOL res;
+  FILETIME t_create;
+  FILETIME t_access;
+  FILETIME t_write;
+  unsigned long long timestamp;
+
+  /* GetFileTime returns FILETIME data which are the number of 100 nanosecs
+     since <Jan 1st 1601>. This function must return the number of seconds
+     since <Jan 1st 1970> */
+
+  unsigned long long offset = 11644473600;
+  /* Number of seconds between <Jan 1st 1601> and <Jan 1st 1970> */
+
+  res = GetFileTime (h, &t_create, &t_access, &t_write);
+
+  timestamp = ((long long)t_write.dwHighDateTime << 32) 
+    + t_write.dwLowDateTime;
+
+  timestamp = (timestamp / 10000000) - offset;
+
+  return (time_t) timestamp;
+}
+#endif
 
 /* Return a GNAT time stamp given a file name.  */
 
@@ -609,9 +685,15 @@ file_time_name (name)
   close (fd);
   return ret;
 
+#elif defined (_WIN32)
+  HANDLE h = CreateFile (name, GENERIC_READ, FILE_SHARE_READ, 0,
+			 OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
+  time_t ret = win32_filetime (h);
+  CloseHandle (h);
+  return ret;
 #else
 
-  int ret = stat (name, &statbuf);
+  int ret = __gnat_stat (name, &statbuf);
 #ifdef VMS
   /* VMS has file versioning */
   return statbuf.st_ctime;
@@ -638,7 +720,7 @@ file_time_fd (fd)
 
   FILESTATUS fs;
   int ret = DosQueryFileInfo (fd, 1, (unsigned char *) &fs,
-				sizeof (FILESTATUS));
+                                sizeof (FILESTATUS));
 
   unsigned file_year  = fs.fdateLastWrite.year;
   unsigned file_month = fs.fdateLastWrite.month;
@@ -686,6 +768,12 @@ file_time_fd (fd)
 
   return tot_secs;
 
+#elif defined (_WIN32)
+  HANDLE h = (HANDLE) _get_osfhandle (fd);
+  time_t ret = win32_filetime (h);
+  CloseHandle (h);
+  return ret;
+
 #else
   struct stat statbuf;
   int ret = fstat (fd, &statbuf);
@@ -713,6 +801,103 @@ get_env_value_ptr (name, len, value)
   return;
 }
 
+/* VMS specific declarations for set_env_value.  */
+
+#ifdef VMS
+
+static char *to_host_path_spec PROTO ((char *));
+
+struct descriptor_s
+{
+  unsigned short len, mbz;
+  char *adr;
+};
+
+typedef struct _ile3
+{
+  unsigned short len, code;
+  char *adr;
+  unsigned short *retlen_adr;
+} ile_s;
+
+#endif
+
+void
+set_env_value (name, value)
+     char *name;
+     char *value;
+{
+#if defined (MSDOS)
+
+#elif defined (VMS)
+  struct descriptor_s name_desc;
+  /* Put in JOB table for now, so that the project stuff at least works */
+  struct descriptor_s table_desc = {7, 0, "LNM$JOB"};
+  char *host_pathspec = to_host_path_spec (value);
+  char *copy_pathspec;
+  int num_dirs_in_pathspec = 1;
+  char *ptr;
+
+  if (*host_pathspec == 0)
+    return;
+
+  name_desc.len = strlen (name);
+  name_desc.mbz = 0;
+  name_desc.adr = name;
+
+  ptr = host_pathspec;
+  while (*ptr++)
+    if (*ptr == ',')
+      num_dirs_in_pathspec++;
+
+  {
+    int i, status;
+    ile_s *ile_array = alloca (sizeof (ile_s) * num_dirs_in_pathspec + 1);
+    char *copy_pathspec = alloca (strlen (host_pathspec) + 1);
+    char *curr, *next;
+
+    strcpy (copy_pathspec, host_pathspec);
+    curr = copy_pathspec;
+    for (i = 0; i < num_dirs_in_pathspec; i++)
+      {
+	next = strchr (curr, ',');
+	if (next == 0)
+	  next = strchr (curr, 0);
+
+	*next = 0;
+	ile_array [i].len = strlen (curr);
+
+	/* Code 2 from lnmdef.h means its a string */
+	ile_array [i].code = 2;
+	ile_array [i].adr = curr;
+
+	/* retlen_adr is ignored */
+	ile_array [i].retlen_adr = 0;
+	curr = next + 1;
+      }
+
+    /* Terminating item must be zero */
+    ile_array [i].len = 0;
+    ile_array [i].code = 0;
+    ile_array [i].adr = 0;
+    ile_array [i].retlen_adr = 0;
+
+    status = LIB$SET_LOGICAL (&name_desc, 0, &table_desc, 0, ile_array);
+    if ((status & 1) != 1)
+      LIB$SIGNAL (status);
+  }
+
+#else
+  int size = strlen (name) + strlen (value) + 2;
+  char *expression;
+
+  expression = (char *) malloc (size * sizeof (char));
+
+  sprintf (expression, "%s=%s", name, value);
+  putenv (expression);
+#endif
+}
+
 #ifdef _WIN32
 
 #include "stdarg.h"
@@ -729,7 +914,7 @@ Get_Libraries_From_Registry ()
 {
   char *result = "";
 
-#if defined (_WIN32) && ! defined (__vxworks) && ! defined (JGNAT)
+#if defined (_WIN32) && ! defined (__vxworks)
 
   HKEY reg_key;
   DWORD name_size, value_size;
@@ -744,7 +929,7 @@ Get_Libraries_From_Registry ()
 
   if (res == ERROR_SUCCESS)
     res = RegOpenKeyExA (reg_key, "Ada Core Technologies", 0,
-			 KEY_READ, &reg_key);
+                         KEY_READ, &reg_key);
 
   if (res == ERROR_SUCCESS)
     res = RegOpenKeyExA (reg_key, "GNAT", 0, KEY_READ, &reg_key);
@@ -758,17 +943,17 @@ Get_Libraries_From_Registry ()
     {
       value_size = name_size = 256;
       res = RegEnumValue (reg_key, index, name, &name_size, 0,
-			  &type, value, &value_size);
+                          &type, value, &value_size);
 
       if (res == ERROR_SUCCESS && type == REG_SZ)
-	{
-	  char *old_result = result;
+        {
+          char *old_result = result;
 
-	  result = (char *) malloc (strlen (old_result) + value_size + 2);
-	  strcpy (result, old_result);
-	  strcat (result, value);
-	  strcat (result, ";");
-	}
+          result = (char *) malloc (strlen (old_result) + value_size + 2);
+          strcpy (result, old_result);
+          strcat (result, value);
+          strcat (result, ";");
+        }
     }
 
   /* Remove the trailing ";".  */
@@ -780,12 +965,43 @@ Get_Libraries_From_Registry ()
 }
 
 int
-file_exists (name)
+__gnat_stat (name, statbuf)
+     char *name;
+     struct stat *statbuf;
+{
+#ifdef _WIN32
+  /* under Windows the directory name for the stat function must not be
+   * terminated by a directory separator expect if just after a drive name
+   */
+  int  name_len  = strlen (name);
+  char last_char = name [name_len - 1];
+  char win32_name [4096];
+
+  strcpy (win32_name, name);
+
+  while ((name_len > 1) && ((last_char == '\\') || (last_char == '/')))
+    {
+      win32_name [name_len - 1] = '\0';
+      name_len  -= 1;
+      last_char = win32_name [name_len - 1];
+    }
+
+  if ((name_len == 2) && (win32_name [1] == ':'))
+    strcat (win32_name, "\\");
+
+  return stat (win32_name, statbuf);
+#else
+  return stat (name, statbuf);
+#endif
+}
+
+int
+__gnat_file_exists (name)
      char *name;
 {
   struct stat statbuf;
 
-  return !stat (name, &statbuf);
+  return !__gnat_stat (name, &statbuf);
 }
 
 int
@@ -795,7 +1011,7 @@ is_regular_file (name)
   int ret;
   struct stat statbuf;
 
-  ret = stat (name, &statbuf);
+  ret = __gnat_stat (name, &statbuf);
   return (!ret && S_ISREG (statbuf.st_mode));
 }
 
@@ -806,7 +1022,7 @@ is_directory (name)
   int ret;
   struct stat statbuf;
 
-  ret = stat (name, &statbuf);
+  ret = __gnat_stat (name, &statbuf);
   return (!ret && S_ISDIR (statbuf.st_mode));
 }
 
@@ -818,7 +1034,7 @@ is_writable_file (name)
   int mode;
   struct stat statbuf;
 
-  ret = stat (name, &statbuf);
+  ret = __gnat_stat (name, &statbuf);
   mode = statbuf.st_mode & S_IWUSR;
   return (!ret && mode);
 }
@@ -826,7 +1042,7 @@ is_writable_file (name)
 #ifdef VMS
 /* Defined in VMS header files */
 #define fork() (decc$$alloc_vfork_blocks() >= 0 ? \
-               lib$get_current_invo_context(decc$$get_vfork_jmpbuf()) : -1)
+               LIB$GET_CURRENT_INVO_CONTEXT (decc$$get_vfork_jmpbuf()) : -1)
 #endif
 
 #if defined (sun) && defined (__SVR4)
@@ -840,20 +1056,25 @@ int
 portable_spawn (args)
     char *args[];
 {
-  int status;
+  int status = 0;
   int finished;
   int pid;
 
-#if defined (__EMX__) || defined (MSDOS) || defined (_WIN32)
-  if (spawnvp (P_WAIT, args [0], args) != 0)
-    return (4);
+#if defined (MSDOS) || defined (_WIN32)
+  status = spawnvp (P_WAIT, args [0], args);
+  if (status < 0) return 4;
+  else return status;
 #elif defined(__vxworks)  /* Mods for VxWorks */
   pid = sp(args[0], args);  /* Spawn process and save pid */
   if (pid == -1)
     return (4);
   while (taskIdVerify(pid) >= 0) {};  /* Wait until spawned task is complete
                                          then continue                       */
-  return 0;
+#else
+#ifdef __EMX__
+  pid = spawnvp (P_NOWAIT, args [0], args);
+  if (pid == -1)
+    return (4);
 #else
   pid = fork ();
   if (pid == -1)
@@ -863,6 +1084,7 @@ portable_spawn (args)
     if (execv (args [0], args) != 0)
       _exit (1);
   }
+#endif;
 
   /* The parent */
 #ifdef VMS
@@ -871,8 +1093,10 @@ portable_spawn (args)
 #else
   finished = wait (&status);
 #endif
-  if (finished != pid || status & 0xffff)
+  if (finished != pid || WIFEXITED (status) == 0)
     return 4;
+
+  return WEXITSTATUS (status);
 #endif
   return 0;
 }
@@ -937,26 +1161,26 @@ void remove_handle (HANDLE h)
   /* -------------------- critical section -------------------- */
   plist_enter();
   pl = PLIST;
-  while (pl) 
+  while (pl)
     {
-      if (pl->h == h) 
-	{
-	  if (pl == PLIST) 
-	    {
-	      PLIST = pl->next;
-	    }
-	  else
-	    {
-	      prev->next = pl->next;
-	    }
-	  free (pl);
-	  break;
-	}
+      if (pl->h == h)
+        {
+          if (pl == PLIST)
+            {
+              PLIST = pl->next;
+            }
+          else
+            {
+              prev->next = pl->next;
+            }
+          free (pl);
+          break;
+        }
       else
-	{
-	  prev = pl;
-	  pl = pl->next;
-	}
+        {
+          prev = pl;
+          pl = pl->next;
+        }
     }
   --plist_length;
   plist_leave();
@@ -1001,17 +1225,21 @@ win32_no_block_spawn (char *command, char *args[])
   }
 
   result = CreateProcess (NULL,
-			  (char *)full_command,
-			  &SA,			 
-			  NULL,
-			  TRUE,
-			  NORMAL_PRIORITY_CLASS,
-			  NULL,
-			  NULL,
-			  &SI,
-			  &PI);
-  add_handle (PI.hProcess);
-  return (int)PI.hProcess;
+                          (char *)full_command,
+                          &SA,
+                          NULL,
+                          TRUE,
+                          NORMAL_PRIORITY_CLASS,
+                          NULL,
+                          NULL,
+                          &SI,
+                          &PI);
+  if (result == TRUE) {
+    add_handle (PI.hProcess);
+    CloseHandle (PI.hThread);
+    return (int)PI.hProcess;
+  } else
+    return -1;
 }
 
 int
@@ -1112,7 +1340,7 @@ portable_wait (process_status)
 
   pid = win32_wait (&status);
 
-#elif defined (__EMX__) || defined (MSDOS) 
+#elif defined (__EMX__) || defined (MSDOS)
   /* ??? See corresponding comment in portable_no_block_spawn. */
 
 #elif defined (__vxworks)
@@ -1165,7 +1393,7 @@ locate_regular_file (file_name, path_val)
      )
     {
       if (is_regular_file (file_name))
-	return xstrdup (file_name);
+        return xstrdup (file_name);
 
       return 0;
     }
@@ -1180,11 +1408,11 @@ locate_regular_file (file_name, path_val)
 
     for (;;)
       {
-	for (; *path_val == pathsep_char ; path_val++)
-	  ;
+        for (; *path_val == pathsep_char ; path_val++)
+          ;
 
       if (*path_val == 0)
-	return 0;
+        return 0;
 
       for (ptr = file_path; *path_val && *path_val != pathsep_char; )
         *ptr++ = *path_val++;
@@ -1215,7 +1443,7 @@ locate_exec (exec_name, path_val)
   if (!strstr (exec_name, EXECUTABLE_SUFFIX))
     {
       char *full_exec_name
-	= alloca (strlen(exec_name) + strlen(EXECUTABLE_SUFFIX) + 1);
+        = alloca (strlen(exec_name) + strlen(EXECUTABLE_SUFFIX) + 1);
 
       strcpy (full_exec_name, exec_name);
       strcat (full_exec_name, EXECUTABLE_SUFFIX);
@@ -1249,75 +1477,112 @@ locate_exec_on_path (exec_name)
    file, directory and path specifications. */
 
 #define MAXNAMES 256
+#define NEW_CANONICAL_FILELIST_INCREMENT 64
 
 static char new_canonical_dirspec [255];
 static char new_canonical_filespec [255];
 static char new_canonical_pathspec [MAXNAMES*255];
-static int new_canonical_filelist_index;
-static char new_canonical_filelist [MAXNAMES][255];
+static unsigned new_canonical_filelist_index;
+static unsigned new_canonical_filelist_in_use;
+static unsigned new_canonical_filelist_allocated;
+static char **new_canonical_filelist;
+static char new_host_pathspec [MAXNAMES*255];
 static char new_host_dirspec [255];
 static char new_host_filespec [255];
 
 /* Routine is called repeatedly by decc$from_vms() via
-   to_canonical_dir_list_init() until it returns 0 or the expansion runs
+   to_canonical_file_list_init() until it returns 0 or the expansion runs
    out. */
 static int
 wildcard_translate_unix (name)
      char *name;
 {
-  char *ext;
+  char *ver;
   char buff [256];
 
   strcpy (buff, name);
-  if (ext = strstr (buff, ".dir."))
+  ver = strrchr (buff, '.');
+
+  /* Chop off the version */
+  if (ver)
+    *ver = 0;
+
+  /* Dynamically extend the allocation by the increment */
+  if (new_canonical_filelist_in_use == new_canonical_filelist_allocated)
     {
-      /* Chop off the extension */
-      *ext = 0;
-      strcpy (new_canonical_filelist[new_canonical_filelist_index++], buff);
+      new_canonical_filelist_allocated += NEW_CANONICAL_FILELIST_INCREMENT;
+      new_canonical_filelist = (char **) realloc
+	(new_canonical_filelist,
+	 new_canonical_filelist_allocated * sizeof (char *));
     }
 
-  if (new_canonical_filelist_index == MAXNAMES)
-    {
-      /* We've run out of room */
-      return 0;
-    }
-  else
-    {
-      /* Keep going */
-      return 1;
-    }
+  new_canonical_filelist[new_canonical_filelist_in_use++] = xstrdup (buff);
+
+  return 1;
 }
 
-/* Translate a wildcard VMS directory spec into a list of Unix directory
+/* Translate a wildcard VMS file spec into a list of Unix file
    specs. First do full translation and copy the results into a list (_init),
-   then return them one at a time (_next) */
+   then return them one at a time (_next). If onlydirs set, only expand
+   directory files. */
 int
-to_canonical_dir_list_init (dirspec)
-     char *dirspec;
+to_canonical_file_list_init (filespec, onlydirs)
+     char *filespec;
+     int onlydirs;
 {
-  int num, len;
+  int len;
   char buff [256];
 
-  strcpy (buff, dirspec);
+  len = strlen (filespec);
+  strcpy (buff, filespec);
 
   /* Only look for directories */
-  strcat (buff, "*.dir");
+  if (onlydirs && !strstr (&buff [len-5], "*.dir"))
+    strcat (buff, "*.dir");
 
-  new_canonical_filelist_index = 0;
-  decc$from_vms(buff, wildcard_translate_unix, 1);	
-  num = new_canonical_filelist_index;
+  decc$from_vms(buff, wildcard_translate_unix, 1);
 
-  /* Reset index in preparation for calls to _next */
-  new_canonical_filelist_index = 0;
-  return num;
+  /* Remove the .dir extension */
+  if (onlydirs)
+    {
+      int i;
+      char *ext;
+
+      for (i=0; i<new_canonical_filelist_in_use; i++)
+	{
+	  ext = strstr (new_canonical_filelist [i], ".dir");
+	  if (ext)
+	    *ext = 0;
+	}
+    }
+
+  return new_canonical_filelist_in_use;
 }
 
+/* Return the next filespec in the list */
 char *
-to_canonical_dir_list_next ()
+to_canonical_file_list_next ()
 {
   return new_canonical_filelist [new_canonical_filelist_index++];
 }
 
+/* Free up storage used in the wildcard expansion */
+void
+to_canonical_file_list_free ()
+{
+  int i;
+
+   for (i=0; i<new_canonical_filelist_in_use; i++)
+     free (new_canonical_filelist [i]);
+
+  free (new_canonical_filelist);
+
+  new_canonical_filelist_in_use = 0;
+  new_canonical_filelist_allocated = 0;
+  new_canonical_filelist_index = 0;
+  new_canonical_filelist = 0;
+}
+
 /* Translate a VMS syntax directory specification in to Unix syntax.
    If prefixflag is set, append an underscore "/". If no indicators
    of VMS syntax found, return input string. Also translate a dirname
@@ -1335,11 +1600,11 @@ to_canonical_dir_spec (dirspec,prefixflag)
       char *dirspec1;
 
       if (strchr (dirspec, ']') || strchr (dirspec, ':'))
-	strcpy (new_canonical_dirspec, (char *)decc$translate_vms (dirspec));
+        strcpy (new_canonical_dirspec, (char *)decc$translate_vms (dirspec));
       else if (!strchr (dirspec, '/') && (dirspec1 = getenv (dirspec)))
-	strcpy (new_canonical_dirspec, (char *)decc$translate_vms (dirspec1));
+        strcpy (new_canonical_dirspec, (char *)decc$translate_vms (dirspec1));
       else
-	strcpy (new_canonical_dirspec, dirspec);
+        strcpy (new_canonical_dirspec, dirspec);
     }
 
   len = strlen (new_canonical_dirspec);
@@ -1387,34 +1652,36 @@ to_canonical_path_spec (pathspec)
     {
       next = strchr (curr, ',');
       if (next == 0)
-	next = strchr (curr, 0);
+        next = strchr (curr, 0);
       strncpy (buff, curr, next - curr);
       buff [next - curr] = 0;
 
       /* Check for wildcards and expand if present */
       if (strchr (buff, '*') || strchr (buff, '%') || strstr (buff, "..."))
-	{
-	  int i, dirs;
+        {
+          int i, dirs;
 
-	  dirs = to_canonical_dir_list_init (buff);
-	  for (i=0; i<dirs; i++)
-	    {
-	      char *next_dir;
+          dirs = to_canonical_file_list_init (buff, 1);
+          for (i=0; i<dirs; i++)
+            {
+              char *next_dir;
 
-	      next_dir = to_canonical_dir_list_next ();
-	      strcat (new_canonical_pathspec, next_dir);
+              next_dir = to_canonical_file_list_next ();
+              strcat (new_canonical_pathspec, next_dir);
 
-	      /* Don't append the separator after the last expansion */
-	      if (i+1 < dirs)
-		strcat (new_canonical_pathspec, ":");
-	    }
-	}
+              /* Don't append the separator after the last expansion */
+              if (i+1 < dirs)
+                strcat (new_canonical_pathspec, ":");
+            }
+
+	  to_canonical_file_list_free ();
+        }
       else
-	{
-	  strcat (new_canonical_pathspec, to_canonical_dir_spec (buff, 0));
-	}
+        {
+          strcat (new_canonical_pathspec, to_canonical_dir_spec (buff, 0));
+        }
       if (*next == 0)
-	break;
+        break;
       strcat (new_canonical_pathspec, ":");
       curr = next + 1;
     }
@@ -1432,12 +1699,50 @@ translate_unix (name, type)
   return 0;
 }
 
+/* Translate a Unix syntax path spec into a VMS style (comma separated
+   list of directories. Only used in this file so make it static */
+
+static char *
+to_host_path_spec (pathspec)
+     char *pathspec;
+{
+  char *curr, *next, buff [256];
+
+  if (pathspec == 0)
+    return pathspec;
+
+  /* Can't very well test for colons, since that's the Unix separator! */
+  if (strchr (pathspec, ']') || strchr (pathspec, ','))
+    return pathspec;
+
+  new_host_pathspec [0] = 0;
+  curr = pathspec;
+
+  for (;;)
+    {
+      next = strchr (curr, ':');
+      if (next == 0)
+        next = strchr (curr, 0);
+
+      strncpy (buff, curr, next - curr);
+      buff [next - curr] = 0;
+
+      strcat (new_host_pathspec, to_host_dir_spec (buff, 0));
+      if (*next == 0)
+        break;
+      strcat (new_host_pathspec, ",");
+      curr = next + 1;
+    }
+
+  return new_host_pathspec;
+}
+
 /* Translate a Unix syntax directory specification into VMS syntax.
    The prefixflag has no effect, but is kept for symmetry with
    to_canonical_dir_spec.
    If indicators of VMS syntax found, return input string. */
 char *
-to_host_dir_spec (dirspec,prefixflag)
+to_host_dir_spec (dirspec, prefixflag)
      char *dirspec;
      int prefixflag;
 {
@@ -1482,7 +1787,7 @@ to_host_file_spec (filespec)
 void
 adjust_os_resource_limits ()
 {
-  sys$adjwsl (131072, 0);
+  SYS$ADJWSL (131072, 0);
 }
 
 #else
@@ -1490,18 +1795,21 @@ adjust_os_resource_limits ()
 /* Dummy functions for Osint import for non-VMS systems */
 
 int
-to_canonical_dir_list_init (dirspec)
+to_canonical_file_list_init (dirspec)
      char *dirspec;
 {
   return 0;
 }
 
 char *
-to_canonical_dir_list_next ()
+to_canonical_file_list_next ()
 {
   return "";
 }
 
+void
+to_canonical_file_list_free () {;}
+
 char *
 to_canonical_dir_spec (dirspec, prefixflag)
      char *dirspec;
@@ -1534,7 +1842,7 @@ to_host_dir_spec (dirspec, prefixflag)
 
 char *
 to_host_file_spec (filespec)
-	char *filespec;
+        char *filespec;
 {
   return filespec;
 }
@@ -1552,17 +1860,22 @@ adjust_os_resource_limits () {;}
 void __dummy () {}
 #endif
 
-
 #if defined (__mips_vxworks)
 int _flush_cache() {
    CACHE_USER_FLUSH (0, ENTIRE_CACHE);
 }
 #endif
 
-#if !(defined (sun) && defined (__SVR4)) && !defined (linux)
+#if defined (CROSS_COMPILE) || \
+(     !(defined (sparc) && defined (sun) && defined (__SVR4)) \
+   && !defined (linux) \
+   && !defined (sgi) \
+   && !defined (hpux) \
+   && !(defined (__alpha__)  && defined (__osf__)) \
+   && !defined (__MINGW32__))
 /* Dummy function to satisfy g-trasym.o.
-   Currently only Solaris & Linux provides a non-dummy version of this
-   procedure */
+   Currently Solaris sparc, HP/UX, IRIX, Linux, Tru64 & Windows provide a
+   non-dummy version of this procedure in libaddr2line.a */
 
 void
 convert_addresses (addrs, n_addr, buf, len)
