@@ -6,7 +6,7 @@
 /*                                                                          */
 /*                          C Implementation File                           */
 /*                                                                          */
-/*                            $Revision: 1.14 $
+/*                            $Revision: 1.15 $
 /*                                                                          */
 /*          Copyright (C) 1992-1999 Free Software Foundation, Inc.          */
 /*                                                                          */
@@ -59,12 +59,10 @@ void put_char_stderr (int c)
 /* Don't use macros on linux since they cause incompatible changes between
    glibc 2.0 and 2.1 */
 #ifdef linux
-#ifdef putchar
-#  undef putchar
-#endif
-#ifdef getchar
-#  undef getchar
-#endif
+#undef putchar
+#undef getchar
+#undef fputc
+#undef stderr
 #endif
 
 int get_int ()
@@ -90,10 +88,7 @@ int get_char (void)
 
 void put_char (int c) { putchar (c); }
 
-void put_char_stderr (int c)
-{
-  fprintf (stderr, "%c", c);
-}
+void put_char_stderr (int c) { fputc (c, stderr); }
 #endif
 
 #ifdef __vxworks
