@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                                   Body                                   */
 /*                                                                          */
-/*                              $Revision: 1.39 $
+/*                              $Revision: 1.46 $
 /*                                                                          */
-/*          Copyright (C) 1992-1999 Free Software Foundation, Inc.          */
+/*          Copyright (C) 1992-2001 Free Software Foundation, Inc.          */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -34,7 +34,7 @@
 
 /* Routines required for implementing routines in Interfaces.C.Streams */
 
-#ifdef __alpha_vxworks
+#ifdef __vxworks
 #include "vxWorks.h"
 #endif
 
@@ -59,7 +59,7 @@ int max_path_len = PATH_MAX;
 #elif defined (linux)
 #include <sys/param.h>
 
-/* If we can't find MAXPATHLEN there, try in unistd.h.  That file might
+/* If we cannot find MAXPATHLEN there, try in unistd.h.  That file might
    not exist, but we'll blow up anyway if we don't try something.  */
 #ifndef MAXPATHLEN
 #include <unistd.h>
@@ -126,6 +126,15 @@ is_regular_file_fd (fd)
   int ret;
   struct stat statbuf;
 
+#ifdef __EMX__
+  /* Programs using screen I/O may need to reset the FPU after
+     initialization of screen-handling related DLL's, so force
+     DLL initialization by doing a null-write and then reset the FPU */
+
+  DosWrite (0, &ret, 0, &ret);
+  __gnat_init_float();
+#endif
+
   ret = fstat (fd, &statbuf);
   return (!ret && S_ISREG (statbuf.st_mode));
 }
@@ -176,89 +185,75 @@ full_name (nam, buffer)
      char *nam;
      char *buffer;
 {
-   char *p;
+  char *p;
 
 #if defined(__EMX__) || defined (__MINGW32__)
-   /* if this is a device file return it as is */
-   /* under Windows NT and OS/2 a device file end with : */
-   if (nam [strlen (nam) - 1] == ':')
-     {
-       strcpy (buffer, nam);
-     }
-     else
-     {
-       _fullpath (buffer, nam, max_path_len);
-       for (p = buffer; *p; p++)
-	 if (*p == '/')
-	   *p = '\\';
-     }
+  /* if this is a device file return it as is */
+  /* under Windows NT and OS/2 a device file end with : */
+  if (nam [strlen (nam) - 1] == ':')
+    {
+      strcpy (buffer, nam);
+    }
+  else
+    {
+      _fullpath (buffer, nam, max_path_len);
+      for (p = buffer; *p; p++)
+	if (*p == '/')
+	  *p = '\\';
+    }
+
+#elif defined (MSDOS)
+  _fixpath (nam, buffer);
+
+#elif defined (sgi)
+
+  /* Use realpath function which resolves links and references to .. and ..
+     on those Unix systems that support it. Note that linux provides it but
+     cannot handle more than 5 symbolic links in a full name, so we use the
+     getcwd approach instead. */
+  realpath (nam, buffer);
+
+#elif defined (VMS)
+  strcpy (buffer, to_canonical_file_spec (nam));
+
+  if (buffer[0] == '/')
+    {
+      strcpy (buffer, to_host_file_spec (buffer));
+    }
+  else
+    {
+      char nambuffer [max_path_len];
+
+      strcpy (nambuffer, buffer);
+      strcpy (buffer, getcwd (buffer, max_path_len, 0));
+      strcat (buffer, "/");
+      strcat (buffer, nambuffer);
+      strcpy (buffer, to_host_file_spec (buffer));
+    }
+
+  return buffer;
 
 #else
-#ifdef MSDOS
-   _fixpath (nam, buffer);
+  extern char *getcwd();
 
-#else
-#if defined (sgi) || defined (sun) || defined (linux)
+  if (nam[0] != '/')
+    {
+      p = getcwd (buffer, max_path_len);
+      if (p == 0)
+	{
+	  buffer[0] = '\0';
+	  return NULL;
+	}
 
-   /* Use realpath function which resolves links and references to .. and ..
-      on those Unix systems that support it. */
-   realpath (nam, buffer);
+      /* If the name returned is an absolute path, it is safe to append /
+	 to the path and concatenate the name of the file. */
+      if (buffer[0] == '/')
+	strcat (buffer, "/");
 
-#else
-#ifdef _WINNT
+      strcat (buffer, nam);
+    }
+  else
+    strcpy (buffer, nam);
 
-   /* The Cygnus development environment supports Unix-like filenames
-      and provides a routine for converting paths to their full
-      specification.  If not using Cygwin32 (i.e. Mingw32), then use
-      the MS routine. */
-#ifdef __CYGWIN32__
-   cygwin32_conv_to_full_win32_path (nam, buffer, max_path_len);
-#else
-   _fullpath (nam, buffer);
-#endif
-
-#else
-#ifdef VMS
-   strcpy (buffer, to_canonical_file_spec (nam));
-
-   if (buffer[0] == '/')
-     {
-       strcpy (buffer, to_host_file_spec (buffer));
-     }
-   else
-     {
-       char nambuffer [max_path_len];
-
-       strcpy (nambuffer, buffer);
-       strcpy (buffer, getcwd (buffer, max_path_len, 0));
-       strcat (buffer, "/");
-       strcat (buffer, nambuffer);
-       strcpy (buffer, to_host_file_spec (buffer));
-     }
-
-   return buffer;
-
-#else
-   extern char *getcwd();
-
-   if (nam[0] != '/')
-     {
-       p = getcwd (buffer, max_path_len);
-       if (p == 0)
-	 {
-	   buffer[0] = '\0';
-	   return NULL;
-	 }
-
-       strcat (buffer, "/");
-       strcat (buffer, nam);
-     }
-   else
-     strcpy (buffer, nam);
-
-#endif
-#endif
-#endif
-#endif
 #endif
 }
