@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                              C Header File                               */
 /*                                                                          */
-/*                            $Revision: 1.38 $                             */
+/*                            $Revision: 1.43 $
 /*                                                                          */
-/*          Copyright (C) 1992-1997 Free Software Foundation, Inc.          */
+/*          Copyright (C) 1992-2001 Free Software Foundation, Inc.          */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -27,14 +27,25 @@
 /****************************************************************************/
 
 /*  Definitions to access front-end functions used in gigi */
-#define Entity_Is_In_Main_Unit lib__entity_is_in_main_unit
-extern Boolean Entity_Is_In_Main_Unit PROTO((Entity_Id));
-#define Defining_Entity		sem_util__defining_entity
+#define In_Extended_Main_Code_Unit lib__in_extended_main_code_unit
+extern Boolean In_Extended_Main_Code_Unit PROTO((Entity_Id));
+#define Get_Encoded_Name exp_dbug__get_encoded_name
+extern void Get_Encoded_Name PROTO((Entity_Id));
+#define Get_External_Name_With_Suffix exp_dbug__get_external_name_with_suffix
+/*  Don't give prototype here for now, as this requires more general 
+    typedefs and macros for fat pointers */
+#define Defining_Entity		 sem_util__defining_entity
 extern Entity_Id Defining_Entity PROTO((Node_Id));
-#define Expr_Value		sem_eval__expr_value
-extern Uint Expr_Value		PROTO((Node_Id));
-#define Expr_Value_S		sem_eval__expr_value_s
-extern Node_Id Expr_Value_S	PROTO((Node_Id));
+#define Requires_Transient_Scope sem_util__requires_transient_scope
+extern Boolean Requires_Transient_Scope PROTO((Entity_Id));
+#define Expr_Value		 sem_eval__expr_value
+extern Uint Expr_Value		 PROTO((Node_Id));
+#define Expr_Value_S		 sem_eval__expr_value_s
+extern Node_Id Expr_Value_S	 PROTO((Node_Id));
+#define Get_Attribute_Definition_Clause \
+sem_ch13__get_attribute_definition_clause
+extern Node_Id Get_Attribute_Definition_Clause PROTO((Entity_Id, char));
+
 
 /* Given GNAT_ENTITY, a GNAT defining identifier node, which denotes some Ada
    entity, this routine returns the equivalent GCC tree for that entity
@@ -102,5 +113,5 @@ extern tree concat_id_with_name PROTO((tree, char *));
 extern tree get_entity_name PROTO((Entity_Id));
 
 /* Return a name for GNAT_ENTITY concatenated with two underscores and
-   STRING.  */
+   SUFFIX.  */
 extern tree create_concat_name PROTO((Entity_Id, char *));
