@@ -4,11 +4,11 @@
 /*                                                                          */
 /*                               A - I N I T                                */
 /*                                                                          */
-/*                            $Revision: 1.130 $                            */
+/*                            $Revision: 1.166 $
 /*                                                                          */
 /*                          C Implementation File                           */
 /*                                                                          */
-/*          Copyright (C) 1992-1999 Free Software Foundation, Inc.          */
+/*          Copyright (C) 1992-2000 Free Software Foundation, Inc.          */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -49,7 +49,6 @@
 
 #include "config.h"
 #include "a-ada.h"
-#include "a-types.h"
 #include "a-raise.h"
 #include <errno.h>
 
@@ -96,6 +95,11 @@ void (*__gl_adafinal_ptr) ()       = 0;
 int  __gl_unreserve_all_interrupts = 0;
 int  __gl_exception_tracebacks     = 0;
 
+/* HAVE_GNAT_INIT_FLOAT must be set on every targets where a __gnat_init_float
+   is defined. If this is not set them a void implementation will be defined
+   at the end of this unit. */
+#undef HAVE_GNAT_INIT_FLOAT
+
 /**********************/
 /* __gnat_set_globals */
 /**********************/
@@ -155,10 +159,10 @@ __gnat_set_globals (main_priority,
 
 #define SA_NODEFER 0
 
-/* AiX doesn't have nanosleep, but provides nsleep instead */
-
 #include <sys/time.h>
 
+/* AiX doesn't have nanosleep, but provides nsleep instead */
+
 int
 nanosleep (Rqtp, Rmtp)
      struct timestruc_t *Rqtp, *Rmtp;
@@ -339,33 +343,114 @@ __gnat_initialize ()
   __gnat_install_handler ();
 }
 
-/* Routines called by 4aemstop.adb.  */
+/* Routines called by 5amastop.adb.  */
 
-#include <machine/reg.h>
+#define SC_GP 29
 
 char *
-_gnat_get_code_loc (context)
+__gnat_get_code_loc (context)
      struct sigcontext *context;
 {
   return (char *) context->sc_pc;
 }
 
 void
-_gnat_enter_handler (context, pc)
+__gnat_enter_handler (context, pc)
      struct sigcontext *context;
      char *pc;
 {
   context->sc_pc = (long) pc;
-  context->sc_regs[EF_GP] = exc_lookup_gp (pc);
+  context->sc_regs[SC_GP] = exc_lookup_gp (pc);
   exc_resume (context);
 }
 
 size_t
-_gnat_machine_state_length ()
+__gnat_machine_state_length ()
 {
   return sizeof (struct sigcontext);
 }
 
+/***********************************/
+/* __gnat_initialize (HPUX version) */
+/***********************************/
+
+#elif defined (hpux)
+
+#include <signal.h>
+
+static void
+__gnat_error_handler (sig)
+     int sig;
+{
+  struct Exception_Data *exception;
+  char *msg;
+
+  switch (sig)
+    {
+    case SIGSEGV:
+      /* FIXME: we need to detect the case of a *real* SIGSEGV */
+      exception = &storage_error;
+      msg = "stack overflow or erroneous memory access";
+      break;
+
+    case SIGBUS:
+      exception = &constraint_error;
+      msg = "SIGBUS";
+      break;
+
+    case SIGFPE:
+      exception = &constraint_error;
+      msg = "SIGFPE";
+      break;
+
+    default:
+      exception = &program_error;
+      msg = "unhandled signal";
+    }
+
+  Raise_From_Signal_Handler (exception, msg);
+}
+
+static void
+__gnat_install_handler ()
+{
+  struct sigaction act;
+
+  /* Set up signal handler to map synchronous signals to appropriate
+     exceptions.  Make sure that the handler isn't interrupted by another
+     signal that might cause a scheduling event! Also setup an alternate
+     stack region for the handler execution so that stack overflows can be
+     handled properly, avoiding a SEGV generation from stack usage by the
+     handler itself. */
+
+  static char handler_stack [SIGSTKSZ];
+
+  stack_t stack;
+
+  stack.ss_sp    = handler_stack;
+  stack.ss_size  = SIGSTKSZ;
+  stack.ss_flags = 0;
+
+  (void) sigaltstack (&stack, NULL);
+
+  act.sa_handler = __gnat_error_handler;
+  act.sa_flags = SA_NODEFER | SA_RESTART | SA_ONSTACK;
+  (void) sigemptyset (&act.sa_mask);
+
+  (void) sigaction (SIGILL,  &act, NULL);
+  (void) sigaction (SIGABRT, &act, NULL);
+  (void) sigaction (SIGFPE,  &act, NULL);
+  (void) sigaction (SIGSEGV, &act, NULL);
+  (void) sigaction (SIGBUS,  &act, NULL);
+}
+
+void
+__gnat_initialize ()
+{
+   __gnat_install_handler ();
+}
+
+
 /*************************************/
 /* __gnat_initialize (Linux version) */
 /*************************************/
@@ -397,8 +482,8 @@ __gnat_error_handler (int sig)
   char *msg;
   static int recurse = 0;
 
-  struct sigcontext_struct *info
-    = (struct sigcontext_struct *) (((char *) &sig) + sizeof (int));
+  struct sigcontext *info
+    = (struct sigcontext *) (((char *) &sig) + sizeof (int));
   /* Linux does not document how to get the machine state in a signal handler,
      but in fact the necessary data is in a sigcontext_struct value that is
      on the stack immediately above the signal number parameter, and the
@@ -659,6 +744,78 @@ __gnat_initialize ()
 
 #elif defined (__INTERIX)
 
+#include <stdio.h>
+#include <signal.h>
+#include <unistd.h>
+
+static void
+__gnat_error_handler (sig)
+     int sig;
+{
+  struct Exception_Data *exception;
+  char *msg;
+
+  switch (sig)
+    {
+    case SIGSEGV:
+      exception = &storage_error;
+      msg = "stack overflow or erroneous memory access";
+      break;
+
+    case SIGBUS:
+      exception = &constraint_error;
+      msg = "SIGBUS";
+      break;
+
+    case SIGFPE:
+      exception = &constraint_error;
+      msg = "SIGFPE";
+      break;
+
+    default:
+      exception = &program_error;
+      msg = "unhandled signal";
+    }
+
+  Raise_From_Signal_Handler (exception, msg);
+}
+
+static void
+__gnat_install_handler ()
+{
+  struct sigaction act;
+
+  /* Set up signal handler to map synchronous signals to appropriate
+     exceptions.  Make sure that the handler isn't interrupted by another
+     signal that might cause a scheduling event! */
+
+  act.sa_handler = __gnat_error_handler;
+  act.sa_flags = 0;
+  (void) sigemptyset (&act.sa_mask);
+
+  /* Handlers for signals besides SIGSEGV cause c974013 to hang */
+/*  (void) sigaction (SIGILL,  &act, NULL); */
+/*  (void) sigaction (SIGABRT, &act, NULL); */
+/*  (void) sigaction (SIGFPE,  &act, NULL); */
+  (void) sigaction (SIGSEGV, &act, NULL);
+/*  (void) sigaction (SIGBUS,  &act, NULL); */
+}
+
+void __gnat_init_float ();
+
+void
+__gnat_initialize ()
+{
+   __gnat_init_float ();
+   __gnat_install_handler ();
+}
+
+/**************************************/
+/* __gnat_initialize (LynxOS version) */
+/**************************************/
+
+#elif defined (__Lynx__)
+
 void __gnat_init_float ();
 
 void
@@ -671,120 +828,13 @@ __gnat_initialize ()
 /* __gnat_initialize (OS/2) */
 /****************************/
 
-/* comment out for now, unusable */
-#elif defined (__EMX__) && 0 /* OS/2 dependent initialization */
+#elif defined (__EMX__) /* OS/2 dependent initialization */
 
-#define INCL_DOS
-#include <os2emx.h>
-/* #include <string.h> */
-/* #include <sys/builtin.h> */
-/* #include <emx/startup.h> */
-#include <emx/syscalls.h>
-#include <signal.h>
-
-ULONG
-_gnat_exception_handler (PEXCEPTIONREPORTRECORD report,
-                      PEXCEPTIONREGISTRATIONRECORD registration,
-                      PCONTEXTRECORD context,
-                      PVOID whatever)
-{
-  __asm__ ("cld");              /* Don't trust */
-
-  if (report->fHandlerFlags & (EH_UNWINDING | EH_EXIT_UNWIND))
-    return XCPT_CONTINUE_SEARCH;
-
-  switch (report->ExceptionNum)
-    {
-    case XCPT_INTEGER_DIVIDE_BY_ZERO:
-      Raise_From_Signal_Handler
-        (&constraint_error, "integer divide by zero");
-      break;
-
-    case XCPT_INTEGER_OVERFLOW:
-      Raise_From_Signal_Handler
-        (&constraint_error, "integer overflow");
-      break;
-
-    case XCPT_ARRAY_BOUNDS_EXCEEDED:
-      Raise_From_Signal_Handler
-        (&constraint_error, "array bounds exceeded");
-      break;
-
-    case XCPT_FLOAT_DENORMAL_OPERAND:
-    case XCPT_FLOAT_DIVIDE_BY_ZERO:
-    case XCPT_FLOAT_INEXACT_RESULT:
-    case XCPT_FLOAT_INVALID_OPERATION:
-    case XCPT_FLOAT_OVERFLOW:
-    case XCPT_FLOAT_STACK_CHECK:
-    case XCPT_FLOAT_UNDERFLOW:
-      Raise_From_Signal_Handler
-        (&constraint_error, "floating point exception");
-      break;
-
-    case XCPT_UNABLE_TO_GROW_STACK:
-      Raise_From_Signal_Handler
-        (&storage_error, "unable to grow stack");
-      break;
-
-    default:
-      break;
-    }
-  return XCPT_CONTINUE_SEARCH;
-}
-
-extern int *gnat_argc;
-extern char **gnat_argv;
-extern char **gnat_envp;
-extern int main(int *argc, char **argv, char **envp);
-extern int gnat_exit_status;
-
-static int __gnat_handler_installed = 0;
-
-/* On OS/2 it is necessary for exception handlers to have an exception handler
-   registration record on the local stack. The reason for this is supporting
-   nested exceptions and unwinding exception handlers. This limitation cannot
-   be worked around, so because __gnat_initialize need to install handlers it
-   cannot return before the actual program has been executed.
-
-   The workaround for this problem takes advantage of the fact that the
-   binder generated main program first stores the passed arguments (argc, argv
-   and envp) and then calls __gnat_initialize. All is needed to make things
-   work is:
-      * on the first call to __gnat_initialize, call main with the stored
-        parameters.
-      * on the next call just return; now the actual program will be executed
-      * in the case main returns instead of terminating, exit with error status
-*/
+void __gnat_init_float ();
 
 void
 __gnat_initialize ()
 {
-   /* When this function gets called, the binder already has stored the argc,
-      argv and envp variables in gnat_argc, gnat_argv and gnat_envp. */
-   ULONG rc;
-   EXCEPTIONREGISTRATIONRECORD xreg = {0, &_gnat_exception_handler};
-
-  /* Do expansion of command-line arguments containing wildcards.
-     This gives same behaviour as in Unix, where the shell handles
-     expansion of wildcards. The function correctly supports all
-     standard OS/2 naming conventions. */
-  _wildcard(&gnat_argc, &gnat_argv);
-
-  /* This routine may only be executed once, see above.*/
-  if (__gnat_handler_installed++) return;
-
-  /* Initialize exception handling. */
-  rc = DosSetExceptionHandler (&xreg);
-  if (rc != 0)
-    DosExit (EXIT_PROCESS, 255);
-
-  /* Return to the program by recursively calling main using the just stored
-     arguments as parameters. This ugly hack is necessary as the exception
-     data must be stored on the stack, which is not possible when returning */
-  gnat_exit_status = main (gnat_argc, gnat_argv, gnat_envp);
-
-  rc = DosUnsetExceptionHandler (&xreg);
-  DosExit (EXIT_PROCESS, rc > 0 ? 255 : gnat_exit_status);
 }
 
 /***********************************/
@@ -928,7 +978,7 @@ __gnat_initialize ()
 /* __gnat_initialize (Solaris and SunOS version) */
 /*************************************************/
 
-#elif defined (sun) && defined (__SVR4)
+#elif defined (sun) && defined (__SVR4) && !defined (__vxworks)
 
 #include <signal.h>
 #include <siginfo.h>
@@ -1147,8 +1197,8 @@ extern struct Exception_Data *Coded_Exception (int);
 #define SS$_STKOVF          1364
 #define SS$_RESIGNAL        2328
 #define MTH$_FLOOVEMAT   1475268       /* Some ACVC_21 CXA tests */
-#define SS$_CE24VRU      3253636       /* Write to unopened file ??? */
-#define SS$_C980VTE      3246436       /* AST requests time slice ??? */
+#define SS$_CE24VRU      3253636       /* Write to unopened file */
+#define SS$_C980VTE      3246436       /* AST requests time slice */
 #define CMA$_EXIT_THREAD 4227492
 #define CMA$_EXCCOPLOS   4228108
 #define CMA$_ALERTED     4227460
@@ -1168,6 +1218,28 @@ __gnat_error_handler (sigargs, mechargs)
   int msg_flag = 0x000f; /* 1 bit for each of the four message parts */
   unsigned short outlen;
 
+  /* Resignaled condtions aren't effected by by pragma Import_Exception */
+
+  switch (sigargs[1])
+  {
+
+    case CMA$_EXIT_THREAD:
+      return SS$_RESIGNAL;
+
+    case SS$_DEBUG: /* Gdb attach, resignal to merge activate gdbstub. */
+      return SS$_RESIGNAL;
+
+    case 1409786: /* Nickerson bug #33 ??? */
+      return SS$_RESIGNAL;
+
+    case 1381050: /* Nickerson bug #33 ??? */
+      return SS$_RESIGNAL;
+
+    case 11829410: /* Resignalled as Use_Error for CE10VRC */
+      return SS$_RESIGNAL;
+
+  }
+
 #ifndef IN_GCC
   /* See if it's an imported exception. Mask off severity bits. */
   exception = Coded_Exception (sigargs [1] & 0xfffffff8);
@@ -1176,8 +1248,8 @@ __gnat_error_handler (sigargs, mechargs)
       msgdesc.len = 256;
       msgdesc.mbz = 0;
       msgdesc.adr = message;
-      sys$getmsg(sigargs[1], &outlen, &msgdesc, msg_flag, 0);
-      message[outlen] = 0;
+      SYS$GETMSG (sigargs[1], &outlen, &msgdesc, msg_flag, 0);
+      message [outlen] = 0;
       msg = message;
 
       exception->Name_Length = 19;
@@ -1215,7 +1287,7 @@ __gnat_error_handler (sigargs, mechargs)
 
       case SS$_HPARITH:
 #ifdef IN_GCC
-	return SS$_RESIGNAL; /* toplev.c handles for compiler... */
+	return SS$_RESIGNAL; /* toplev.c handles for compiler */
 #else
 	{
 	  exception = &constraint_error;
@@ -1239,21 +1311,6 @@ __gnat_error_handler (sigargs, mechargs)
 	msg = "";
 	break;
 
-      case SS$_DEBUG: /* Gdb attach, resignal to merge activate gdbstub. */
-	return SS$_RESIGNAL;
-
-      case CMA$_EXIT_THREAD:
-	return SS$_RESIGNAL;
-
-      case 1409786: /* Nickerson bug #33 ??? */
-	return SS$_RESIGNAL;
-
-      case 1381050: /* Nickerson bug #33 ??? */
-	return SS$_RESIGNAL;
-
-      case 11829410: /* Resignalled as Use_Error for CE10VRC */
-        return SS$_RESIGNAL;
-
       default:
 #ifdef IN_GCC
 	exception = &program_error;
@@ -1264,8 +1321,8 @@ __gnat_error_handler (sigargs, mechargs)
 	msgdesc.len = 256;
 	msgdesc.mbz = 0;
 	msgdesc.adr = message;
-	sys$getmsg(sigargs[1], &outlen, &msgdesc, msg_flag, 0);
-	message[outlen] = 0;
+	SYS$GETMSG (sigargs[1], &outlen, &msgdesc, msg_flag, 0);
+	message [outlen] = 0;
 	msg = message;
 #endif
 	break;
@@ -1285,7 +1342,7 @@ install_handler ()
   __gnat_error_prehandler_stack = &c[1024];
 
   /* __gnat_error_prehandler is an assembly function.  */
-  sys$setexv (1, __gnat_error_prehandler, 3, &prvhnd);
+  SYS$SETEXV (1, __gnat_error_prehandler, 3, &prvhnd);
 }
 
 void
@@ -1304,9 +1361,7 @@ __gnat_initialize()
 #include <signal.h>
 #include <taskLib.h>
 #include <intLib.h>
-#ifdef __mips_vxworks
-#include <arch/mips/ivMips.h>
-#endif
+#include <iv.h>
 
 static void
 __gnat_int_handler (interr)
@@ -1325,12 +1380,21 @@ __gnat_int_handler (interr)
 
 void *__gnat_stack_limit = NULL;
 
-/* getpid is used by s-parint.adb */
+#ifndef __alpha_vxworks
+/* getpid is used by s-parint.adb, but is not defined by VxWorks, except
+   on Alpha VxWorks */
 
 long getpid (void)
 {
   return taskIdSelf ();
 }
+#endif
+
+/* This is needed by the GNAT run time to handle Vxworks interrupts */
+int __gnat_inum_to_ivec (int num)
+{
+  return INUM_TO_IVEC (num);
+}
 
 static void
 __gnat_error_handler (sig, code, sc)
@@ -1405,10 +1469,12 @@ __gnat_install_handler ()
   (void) sigaction (SIGBUS,  &act, NULL);
 }
 
+#define HAVE_GNAT_INIT_FLOAT
+
 void
 __gnat_init_float ()
 {
-#ifdef _ARCH_PPC
+#if defined (_ARCH_PPC) && !defined (_SOFT_FLOAT)
   /* Disable overflow/underflow exceptions on the PPC processor, this is needed
       to get correct Ada semantic */
   asm ("mtfsb0 25");
@@ -1465,12 +1531,15 @@ __gnat_initialize ()
 /* __gnat_init_float */
 /*********************/
 
-/* This routine is NT specific, it is called as each process thread
-   is created, for possible initialization of the FP processor. It is
-   defined in all Win32 versions, but only actually does something on
-   the i386 version, where it properly initializes the FPU for GNAT use.  */
+/* This routine is called as each process thread is created, for possible
+   initialization of the FP processor. This version is used under INTERIX,
+   WIN32 and could be used under OS/2 */
+
+#if defined (_WIN32) || defined (__INTERIX) || defined (__EMX__) \
+  || defined (__Lynx__)
+
+#define HAVE_GNAT_INIT_FLOAT
 
-#if defined (_WIN32) || defined (__INTERIX)
 void
 __gnat_init_float ()
 {
@@ -1479,50 +1548,22 @@ __gnat_init_float ()
   /* This is used to properly initialize the FPU on an x86 for each
      process thread. */
 
-/* ??? These bits need to be documented.  */
-#define FPU_RESERVED    0xF0C0
-#define FPU_DEFAULT     0x033F
-
-  volatile unsigned short cw;
-
-  /* Get fp control word */
-  __asm__ volatile ("fnstcw %0" : "=m" (cw) : );
-
-  /* Do initialization masking.  */
-  cw &= FPU_RESERVED;
-  cw |= FPU_DEFAULT;
-
-  /* Set the control word.  */
-  __asm__ volatile ("fldcw %0" :: "m" (cw));
+  asm ("finit");
 
 #endif  /* Defined __i386__ */
 }
 #endif
 
-/* When compiling JGNAT "tree.h" is an empty file */
-
-#include "tree.h"
-
-/* We need a dummy update_path for GCC 2.7.2 and for JGNAT. */
-
-#if !defined (DECL_ORIGINAL_TYPE)
-
-char *
-update_path (path, key)
-     char *path, *key;
-{
-  return path;
-}
-
-#endif
-
 /* Get the stack unwinding mechanism when available and when compiling
    a-init.c for the run time. Except in the case of a restricted run-time,
-   such as RT-Linux modules (__RT__ is defined). Also no stack unwinding
-   is needed when compiling JGNAT. */
+   such as RT-Linux modules (__RT__ is defined). */
 
 #if !defined (IN_GCC) && !defined (__RT__)
-#include "defaults.h"
+/* If we have a definition of INCOMING_RETURN_ADDR_RTX, assume that
+   the rest of the DWARF 2 frame unwind support is also provided.  */
+#if !defined (DWARF2_UNWIND_INFO) && defined (INCOMING_RETURN_ADDR_RTX)
+#define DWARF2_UNWIND_INFO 1
+#endif
 
 #ifdef DWARF2_UNWIND_INFO
 #ifndef _SIZE_T_DEFINED
@@ -1627,7 +1668,7 @@ get_return_addr (frame_state *udata, frame_state *sub_udata)
 
 /* Thread-safe version of __frame_state_for */
 
-struct frame_state *
+static struct frame_state *
 __frame_state_for_r (void *pc_target, struct frame_state *state_in)
 {
   struct frame_state *f;
@@ -1794,38 +1835,10 @@ __gnat_get_code_loc (m)
 #endif /* DWARF2_UNWIND_INFO */
 #endif /* !IN_GCC && !__RT__ */
 
-#ifndef DWARF2_UNWIND_INFO
-/* We are either compiling gcc or there is no dwarf2 unwind mechanism */
-__SIZE_TYPE__
-__gnat_machine_state_length ()
-{
-  return 0;
-}
-
-void *
-__gnat_get_code_loc (m)
-     void *m;
-{
-  return NULL;
-}
-
+#ifndef HAVE_GNAT_INIT_FLOAT
+/* All targets without a specific __gnat_init_float will use an empty one */
 void
-__gnat_set_machine_state (m)
-     void *m;
+__gnat_init_float ()
 {
 }
-
-void
-__gnat_pop_frame (m)
-     void *m;
-{
-}
-
-void
-__gnat_enter_handler (m, handler)
-     void *m;
-     void *handler;
-{
-  return;
-}
-#endif /* !DWARF2_UNWIND_INFO */
+#endif
