@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                           C Implementation File                          */
 /*                                                                          */
-/*                             $Revision: 1.153 $
+/*                             $Revision: 1.169 $
 /*                                                                          */
-/*          Copyright (C) 1992-1999 Free Software Foundation, Inc.          */
+/*          Copyright (C) 1992-2000 Free Software Foundation, Inc.          */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -65,6 +65,8 @@
 extern char *xmalloc ();
 extern char *main_input_filename;
 extern FILE *asm_out_file;
+extern int save_argc;
+extern char **save_argv;
 
 /* Tables describing GCC tree codes used only by GNAT.  
 
@@ -165,14 +167,10 @@ yyparse ()
 void 
 init_gnat_args ()
 {
-  extern int save_argc;
-  extern char **save_argv;
-
-  /* initialize gnat_argv with save_argv size */
+  /* Initialize gnat_argv with save_argv size */
   gnat_argv = (char **) xmalloc ((save_argc + 1) * sizeof (gnat_argv[0])); 
 
   gnat_argv [0] = save_argv[0];     /* name of the command */ 
-
   gnat_argc = 1;
 }
 
@@ -185,12 +183,10 @@ int
 lang_decode_option (p)
      char *p;
 {
-  extern int  save_argc;
-  extern char **save_argv;
-
   int i;
 
-  if (!gnat_argc) init_gnat_args ();
+  if (! gnat_argc)
+    init_gnat_args ();
 
   if (!strncmp (p, "-I", 2))
     {
@@ -200,9 +196,19 @@ lang_decode_option (p)
       return 1;
     }
 
+  if (!strncmp (p, "-gant", 5))
+    {
+      char *q = (char *) xmalloc (strlen (p) + 1);
+
+      warning ("`-gnat' misspelled as `-gant'");
+      strcpy (q, p);
+      q[2] = 'n', q[3] = 'a';
+      p = q;
+    }
+
   if (!strncmp (p, "-gnat", 5))
     {
-      /* recopy the switches without the 'gnat' prefix */
+      /* Recopy the switches without the 'gnat' prefix */
 
       gnat_argv[gnat_argc] =  (char *) xmalloc (strlen (p) - 3);
       gnat_argv[gnat_argc][0] = '-';
@@ -276,7 +282,6 @@ fatal_function (msgid, ap)
 void
 lang_init ()
 {
-
   if (!gnat_argc)
     init_gnat_args ();
 
@@ -412,8 +417,9 @@ print_lang_identifier (file, node, indent)
      int indent;
 {}
 
-/* Expands GNAT-specific GCC tree nodes.  The only ones we support here are
-   TRANSFORM_EXPR, UNCHECKED_CONVERT_EXPR, USE_EXPR and NULL_EXPR.  */
+/* Expands GNAT-specific GCC tree nodes.  The only ones we support
+   here are TRANSFORM_EXPR, UNCHECKED_CONVERT_EXPR, ALLOCATE_EXPR,
+   USE_EXPR and NULL_EXPR.  */
 
 static rtx
 gnat_expand_expr (exp, target, tmode, modifier)
@@ -458,7 +464,7 @@ gnat_expand_expr (exp, target, tmode, modifier)
 
       /* If we're converting between an aggregate and non-aggregate type
 	 and we have a MEM TARGET, we can't use it, since MEM_IN_STRUCT_P
-	 would be set incorrect.  */
+	 would be set incorrectly.  */
       if (target != 0 && GET_CODE (target) == MEM
 	  && (MEM_IN_STRUCT_P (target) != AGGREGATE_TYPE_P (inner_type)))
 	target = 0;
@@ -466,11 +472,31 @@ gnat_expand_expr (exp, target, tmode, modifier)
       /* If the input and output are both the same mode (usually BLKmode),
 	 just return the expanded input since we want just the bits.  But
 	 we can't do this if the output is more strictly aligned than
-	 the input.  */
+	 the input or if the type is BLKmode and the sizes differ.  */
       if (TYPE_MODE (type) == TYPE_MODE (inner_type)
 	  && (TYPE_ALIGN (type) <= TYPE_ALIGN (inner_type)
-	      || TYPE_ALIGN (inner_type) >= BIGGEST_ALIGNMENT))
-	new = TREE_OPERAND (exp, 0);
+	      || TYPE_ALIGN (inner_type) >= BIGGEST_ALIGNMENT)
+	  && ! (TYPE_MODE (type) == BLKmode
+		&& ! operand_equal_p (TYPE_SIZE (type),
+				      TYPE_SIZE (inner_type), 0)))
+	{
+	  new = TREE_OPERAND (exp, 0);
+
+	  /* If the new type is less strictly aligned than the inner type,
+	     make a new type with the less strict alignment just for
+	     code generation purposes of this node.  If it is a decl,
+	     we can't change the type, so make a NOP_EXPR.  */
+	  if (TYPE_ALIGN (type) != TYPE_ALIGN (inner_type))
+	    {
+	      tree copy_type = copy_node (inner_type);
+
+	      TYPE_ALIGN (copy_type) = TYPE_ALIGN (type);
+	      if (TREE_CODE_CLASS (TREE_CODE (new)) == 'd')
+		new = build1 (NOP_EXPR, copy_type, new);
+	      else
+		TREE_TYPE (new) = copy_type;
+	    }
+	}
 
       /* If either mode is BLKmode, memory will be involved, so do this
 	 via pointer punning.  Likewise, this doesn't work if there
@@ -557,6 +583,16 @@ gnat_expand_expr (exp, target, tmode, modifier)
       else
 	return gen_reg_rtx (TYPE_MODE  (type));
 
+    case ALLOCATE_EXPR:
+      if (TREE_CODE (TREE_OPERAND (exp, 1)) != INTEGER_CST)
+	gigi_abort (205);
+
+      return
+	allocate_dynamic_stack_space
+	  (expand_expr (TREE_OPERAND (exp, 0), NULL_RTX, TYPE_MODE (sizetype),
+			EXPAND_NORMAL),
+	   NULL_RTX, TREE_INT_CST_LOW (TREE_OPERAND (exp, 1)));
+
     case USE_EXPR:
       if (target != const0_rtx)
 	gigi_abort (203);
@@ -622,6 +658,9 @@ update_setjmp_buf (buf)
   if (HAVE_save_stack_nonlocal)
     sa_mode = insn_operand_mode[(int) CODE_FOR_save_stack_nonlocal][0];
 #endif
+#ifdef STACK_SAVEAREA_MODE
+  sa_mode = STACK_SAVEAREA_MODE (sa_mode, SAVE_NONLOCAL);
+#endif
 
   stack_save
     = gen_rtx (MEM, sa_mode,
@@ -640,6 +679,65 @@ update_setjmp_buf (buf)
   emit_stack_save (SAVE_NONLOCAL, &stack_save, NULL_RTX);
 }
 
+/* See if DECL has an RTL that is indirect via a pseudo-register or a
+   memory location and replace it with an indirect reference if so.
+   This improves the debugger's ability to display the value.  */
+
+void
+adjust_decl_rtl (decl)
+     tree decl;
+{
+  tree new_type;
+
+  /* If this decl is already indirect, don't do anything.  This should
+     mean that the decl cannot be indirect, but there's no point in
+     adding an abort to check that.  */
+  if (TREE_CODE (decl) != CONST_DECL
+      && ! DECL_BY_REF_P (decl)
+      && (GET_CODE (DECL_RTL (decl)) == MEM
+	  && (GET_CODE (XEXP (DECL_RTL (decl), 0)) == MEM
+	      || (GET_CODE (XEXP (DECL_RTL (decl), 0)) == REG
+		  && (REGNO (XEXP (DECL_RTL (decl), 0))
+		      > LAST_VIRTUAL_REGISTER))))
+      /* We can't do this if the reference type's mode is not the same
+	 as the current mode, which means this may not work on mixed 32/64
+	 bit systems.  */
+      && (new_type = build_reference_type (TREE_TYPE (decl))) != 0
+      && TYPE_MODE (new_type) == GET_MODE (XEXP (DECL_RTL (decl), 0))
+      /* If this is a PARM_DECL, we can only do it if DECL_INCOMING_RTL
+	 is also an indirect and of the same mode and if the object is
+	 readonly, the latter condition because we don't want to upset the
+	 handling of CICO_LIST.  */
+      && (TREE_CODE (decl) != PARM_DECL
+	  || (GET_CODE (DECL_INCOMING_RTL (decl)) == MEM
+	      && (TYPE_MODE (new_type)
+		  == GET_MODE (XEXP (DECL_INCOMING_RTL (decl), 0)))
+	      && TREE_READONLY (decl))))
+    {
+      new_type = build_type_variant (new_type, 1, 0);
+
+      DECL_POINTS_TO_READONLY_P (decl) = TREE_READONLY (decl);
+      DECL_BY_REF_P (decl) = 1;
+      DECL_RTL (decl) = XEXP (DECL_RTL (decl), 0);
+      TREE_TYPE (decl) = new_type;
+      DECL_MODE (decl) = TYPE_MODE (new_type);
+      DECL_ALIGN (decl) = TYPE_ALIGN (new_type);
+      DECL_SIZE (decl) = TYPE_SIZE (new_type);
+
+      if (TREE_CODE (decl) == PARM_DECL)
+	DECL_INCOMING_RTL (decl) = XEXP (DECL_INCOMING_RTL (decl), 0);
+
+      /* If DECL_INITIAL was set, it should be updated to show that
+	 the decl is initialized to the address of that thing.
+	 Otherwise, just set it to the address of this decl.
+	 It needs to be set so that GCC does not think the decl is
+	 unused.  */
+      DECL_INITIAL (decl)
+	= build1 (ADDR_EXPR, new_type,
+		  DECL_INITIAL (decl) != 0 ? DECL_INITIAL (decl) : decl);
+    }
+}
+
 /* Record the current code position in GNAT_NODE.  */
 
 void
@@ -647,10 +745,15 @@ record_code_position (gnat_node)
      Node_Id gnat_node;
 {
   if (global_bindings_p ())
-    save_gnu_tree (gnat_node, get_elaboration_location (), 1);
+    {
+      /* Make a dummy entry so multiple things at the same location don't
+	 end up in the same place.  */
+      add_pending_elaborations (NULL_TREE, NULL_TREE);
+      save_gnu_tree (gnat_node, get_elaboration_location (), 1);
+    }
   else
     /* Always emit another insn in case marking the last insn
-       addressable needs some fixups.  */
+       addressable needs some fixups and also for above reason.  */
     save_gnu_tree (gnat_node,
 		   (tree) emit_note (NULL_PTR, NOTE_INSN_DELETED), 1);
 }
@@ -755,14 +858,28 @@ int
 default_pass_by_ref (gnu_type)
      tree gnu_type;
 {
-  /* We pass aggregates by reference if they are sufficiently large.
-     The choice of constant here is somewhat arbitrary.  */
-  return (AGGREGATE_TYPE_P (gnu_type)
-	  && (TREE_CODE (TYPE_SIZE (gnu_type)) != INTEGER_CST
-	      || TREE_INT_CST_HIGH (TYPE_SIZE (gnu_type)) != 0
-	      || (TREE_INT_CST_LOW (TYPE_SIZE (gnu_type))
-		  > (HOST_WIDE_INT) 8 * BITS_PER_WORD)
-	      || TREE_OVERFLOW (TYPE_SIZE (gnu_type))));
+  CUMULATIVE_ARGS cum;
+
+  INIT_CUMULATIVE_ARGS (cum, NULL_TREE, NULL_TREE, 0);
+
+  /* We pass aggregates by reference if they are sufficiently large.  The
+     choice of constant here is somewhat arbitrary.  We also pass by
+     reference if the target machine would either pass or return by
+     reference.  Strictly speaking, we need only check the return if this
+     is an In Out parameter, but it's probably best to err on the side of
+     passing more things by reference.  */
+  return (0
+#ifdef FUNCTION_ARG_PASS_BY_REFERENCE
+	  || FUNCTION_ARG_PASS_BY_REFERENCE (cum, TYPE_MODE (gnu_type),
+					     gnu_type, 1)
+#endif
+	  || RETURN_IN_MEMORY (gnu_type)
+	  || (AGGREGATE_TYPE_P (gnu_type)
+	      && (TREE_CODE (TYPE_SIZE (gnu_type)) != INTEGER_CST
+		  || TREE_INT_CST_HIGH (TYPE_SIZE (gnu_type)) != 0
+		  || (TREE_INT_CST_LOW (TYPE_SIZE (gnu_type))
+		      > (HOST_WIDE_INT) 8 * TYPE_ALIGN (gnu_type))
+		  || TREE_OVERFLOW (TYPE_SIZE (gnu_type)))));
 }
 
 /* GNU_TYPE is the type of a subprogram parameter.  Determine from the type if
