@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                              C Header File                               */
 /*                                                                          */
-/*                            $Revision: 1.31 $                             */
+/*                            $Revision: 1.34 $
 /*                                                                          */
-/*          Copyright (C) 1992-1999 Free Software Foundation, Inc.          */
+/*          Copyright (C) 1992-2001 Free Software Foundation, Inc.          */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -84,8 +84,9 @@ Get_Decoded_Name_String (Id)
   return Name_Buffer;
 }
 
-/* Likewise, but also upper-case the string unless it is a character
-   literal.  This is used for building the enumeration literal table */
+/* Like Get_Decoded_Name_String, but the result has all qualification and
+   package body entity suffixes stripped, and also all letters are upper
+   cased.  This is used fo rbuilding the enumeration literal table. */
 
 extern void casing__set_all_upper_case PROTO ((void));
 
@@ -93,7 +94,7 @@ INLINE char *
 Get_Upper_Decoded_Name_String (Id)
      Name_Id Id;
 {
-  namet__get_decoded_name_string (Id);
+  namet__get_unqualified_decoded_name_string (Id);
   if (Name_Buffer [0] != '\'')
     casing__set_all_upper_case ();
   Name_Buffer [Name_Len] = 0;
@@ -128,7 +129,7 @@ extern File_Name_Type Debug_Source_Name	PROTO((Source_File_Index));
 extern File_Name_Type Reference_Name	PROTO((Source_File_Index));
 #define Get_Source_File_Index sinput__get_source_file_index
 extern Source_File_Index Get_Source_File_Index PROTO((Source_Ptr));
-#define Get_Line_Number sinput__get_line_number
+#define Get_Logical_Line_Number sinput__get_logical_line_number
 extern Logical_Line_Number Get_Line_Number PROTO((Source_Ptr));
 
 
