@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                          C Implementation File                           */
 /*                                                                          */
-/*                            $Revision: 1.46 $                             */
+/*                            $Revision: 1.61 $
 /*                                                                          */
-/*          Copyright (C) 1992-1999 Free Software Foundation, Inc.          */
+/*       Copyright (C) 1992-1999-2000 Free Software Foundation, Inc.        */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -35,17 +35,15 @@
 /* This file contains system dependent symbols that are referenced in the
    GNAT Run Time Library */
 
-#ifdef __alpha_vxworks
+#ifdef __vxworks
 #include "vxWorks.h"
 #endif
-
 #include "config.h"
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <errno.h>
 #include <stdio.h>
-
 /*
    mode_read_text
    open text file for reading
@@ -136,7 +134,6 @@
 */
 
 #if defined(WINNT) || defined (MSDOS) || defined (__EMX__)
-
 const char *mode_read_text = "rt";
 const char *mode_write_text = "wt";
 const char *mode_append_text = "at";
@@ -151,21 +148,21 @@ const char *mode_write_binary_plus = "w+b";
 const char *mode_append_binary_plus = "a+b";
 const char text_translation_required = 1;
 
-/* For now these functions do nothing, must be fixed later??? */
 void
 set_binary_mode (handle)
      int handle;
 {
+  _setmode (handle, O_BINARY);
 }
 
 void
 set_text_mode (handle)
      int handle;
 {
+  _setmode (handle, O_TEXT);
 }
 
 #ifdef __MINGW32__
-
 #include <windows.h>
 
 /* Return the name of the tty.   Under windows there is no name for
@@ -185,23 +182,61 @@ ttyname (filedes)
 /* This function is needed to fix a bug under Win95/98. Under these plateforms
    doing :
                 ch1 = getch();
-		ch1 = fgetc (stdin);
+		ch2 = fgetc (stdin);
+
    will put the same character into ch1 and ch2. It seem that the character
    read by getch() is not correctly removed from the buffer. Even a
    fflush(stdin) does not fix the bug. This bug does not appear under Window
-   NT but we call winflush anyway because it would be a big overhead to test
-   (at runtime) each time if we are running under 95/98 or NT.
+   NT. So we have two version of this routine below one for 95/98 and one for
+   NT/2000 version of Windows. There is also a special routine (winflushinit)
+   that will be called only the first time to check which version of Windows
+   we are running running on to set the right routine to use.
 
-   Calling FlushConsoleInputBuffer just after getch() fix the bug.
- */
-void 
-winflush (void)
-{
+   This problem occurs when using Text_IO.Get_Line after Text_IO.Get_Immediate
+   for example.
+
+   Calling FlushConsoleInputBuffer just after getch() fix the bug under 
+   95/98. */
+
+void winflush_init (void);
+
+void winflush_95 (void);
+
+void winflush_nt (void);
+
+/* winflusfunction is set first to the winflushinit function which will check
+   the OS version 95/98 or NT/2000 */
+
+void (*winflush_function)(void) = winflush_init;
+
+/* This function does the runtime check of the OS version and then sets
+   winflush_function to the appropriate function and then call it. */ 
+
+void
+winflush_init (void)
+{ 
+  DWORD dwVersion = GetVersion();
+
+  if (dwVersion < 0x80000000)                /* Windows NT/2000 */
+    winflush_function = winflush_nt;
+  else                                       /* Windows 95/98   */
+    winflush_function = winflush_95;
+
+  (*winflush_function)();      /* Perform the 'flush' */
+
+}
+
+void winflush_95 (void)
+{ 
   FlushConsoleInputBuffer (GetStdHandle (STD_INPUT_HANDLE));
 }
 
-#endif
+void winflush_nt (void)
+{
+  /* does nothing as there is no problem under NT */
+}
 
+#endif
 #else
 
 const char *mode_read_text = "r";
@@ -235,14 +270,14 @@ set_text_mode (stream)
 #endif
 
 #if defined (linux) || defined (sun) || defined (sgi) || defined (__EMX__) \
-  || (defined (__osf__) && ! defined (__alpha_vxworks)) || defined (WINNT)
+  || (defined (__osf__) && ! defined (__alpha_vxworks)) || defined (WINNT) \
+  || defined (__MACHTEN__)
 #include <termios.h>
 #include <fcntl.h>
 #elif defined (VMS)
 extern char *decc$ga_stdscr;
 static int initted = 0;
 #endif
-
 /* Implements the common processing for getc_immediate and
    getc_immediate_nowait. */
 
@@ -285,7 +320,7 @@ getc_immediate_common (stream, ch, end_of_file, avail, waiting)
 {
 #if defined (linux) || defined (sun) || defined (sgi) || defined (__EMX__) \
     || (defined (__osf__) && ! defined (__alpha_vxworks)) \
-    || defined (__CYGWIN32__)
+    || defined (__CYGWIN32__) || defined (__MACHTEN__)
   char c;
   int nread;
   int good_one = 0;
@@ -302,7 +337,7 @@ getc_immediate_common (stream, ch, end_of_file, avail, waiting)
           /* Set RAW mode */
           termios_rec.c_lflag = termios_rec.c_lflag & ~ICANON;
 #if defined(sgi) || defined (sun) || defined (__EMX__) || defined (__osf__) \
-      || defined (linux)
+      || defined (linux) || defined (__MACHTEN__)
           eof_ch = termios_rec.c_cc[VEOF];
 
           /* If waiting (i.e. Get_Immediate (Char)), set MIN = 1 and wait for
@@ -394,7 +429,7 @@ getc_immediate_common (stream, ch, end_of_file, avail, waiting)
       if (waiting)
 	{
 	  *ch = getch();
-	  winflush(); /* see comments before winflush definition */
+	  (*winflush_function)();
 
 	  if (*ch == eot_ch)
 	    *end_of_file = 1;
@@ -411,7 +446,7 @@ getc_immediate_common (stream, ch, end_of_file, avail, waiting)
 	    {
 	      *avail = 1;
 	      *ch = getch();
-	      winflush(); /* see comments before winflush definition */
+	      (*winflush_function)();
 
 	      if (*ch == eot_ch)
 		*end_of_file = 1;
@@ -461,7 +496,6 @@ char *rts_get_lpCommandLine (void) { return (GetCommandLineA ()); }
 int   rts_get_nShowCmd      (void) { return (1); }
 
 #endif /* WINNT */
-
 #ifdef VMS
 
 /* This gets around a problem with using the old threads library on VMS 7.0. */
@@ -478,7 +512,58 @@ get_gmtoff ()
   return ts->tm_gmtoff;
 }
 #endif
+/* Definition of __gnat_locatime_r used by a-calend.adb */
 
+#if defined (_AIX) || defined (__EMX__) || defined (__INTERIX)
+#include <string.h>
+#include <time.h>
+#define Lock_Task system__soft_links__lock_task
+extern void (*Lock_Task) (void);
+
+#define Unlock_Task system__soft_links__unlock_task
+extern void (*Unlock_Task) (void);
+
+/* Provide reentrant version of localtime on Aix and OS/2. Note that AiX does
+   provide localtime_r, but in the library libc_r which doesn't get included
+   systematically, so we can't use it. */
+
+struct tm *
+__gnat_localtime_r (const time_t *timer, struct tm *tp)
+{
+  struct tm *tmp;
+
+  (*Lock_Task) ();
+  tmp = localtime (timer);
+  memcpy (tp, tmp, sizeof (struct tm));
+  (*Unlock_Task) ();
+  return tp;
+}
+
+#elif defined (__Lynx__)
+#include <time.h>
+
+/* LynxOS provides a non standard localtime_r */
+
+struct tm *
+__gnat_localtime_r (const time_t *timer, struct tm *tp)
+{
+  return localtime_r (tp, timer);
+}
+
+#elif defined (VMS) || defined (__MINGW32__)
+/* __gnat_localtime_r is not needed on NT and VMS */
+
+#else
+#include <time.h>
+
+/* All other targets provide a standard localtime_r */
+
+struct tm *
+__gnat_localtime_r (const time_t *timer, struct tm *tp)
+{
+  return (struct tm *)localtime_r (timer, tp);
+}
+#endif
 #if 0
 #if defined(_X86_) && defined(__MINGW32__)
 /* this could be used by other x86 OS like OS/2, Linux, Solaris x86 */
