@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                              C Header File                               */
 /*                                                                          */
-/*                            $Revision: 1.121 $
+/*                            $Revision: 1.126 $
 /*                                                                          */
-/*          Copyright (C) 1992-1999 Free Software Foundation, Inc.          */
+/*          Copyright (C) 1992-2000 Free Software Foundation, Inc.          */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -133,14 +133,14 @@ extern int force_global;
 /* For FUNCTION_TYPEs, nonzero if the function returns by reference.  */
 #define TYPE_RETURNS_BY_REF_P(NODE) TYPE_LANG_FLAG_4 (NODE)
 
-/* For VOID_TYPE, UNION_TYPE, and RECORD_TYPE, nonzero if this is a dummy
-   type, made to correspond to a private or incomplete type.  */
+/* For VOID_TYPE, ENUMERAL_TYPE, UNION_TYPE, and RECORD_TYPE, nonzero if this
+   is a dummy type, made to correspond to a private or incomplete type.  */
 #define TYPE_DUMMY_P(NODE) TYPE_LANG_FLAG_4 (NODE)
 
 /* True if TYPE is such a dummy type.  */
 #define TYPE_IS_DUMMY_P(NODE) \
   ((TREE_CODE (NODE) == VOID_TYPE || TREE_CODE (NODE) == RECORD_TYPE	\
-    || TREE_CODE (NODE) == UNION_TYPE)					\
+    || TREE_CODE (NODE) == UNION_TYPE || TREE_CODE (NODE) == ENUMERAL_TYPE) \
    && TYPE_DUMMY_P (NODE))
 
 /* Nonzero if this corresponds to a type where alignment is guaranteed
@@ -442,8 +442,11 @@ extern void finish_record_type		PROTO((tree, tree, int, int));
    PARM_DECL nodes that are the subprogram arguments.  CICO_LIST is the
    copy-in/copy-out list to be stored into TYPE_CI_CO_LIST. 
    RETURNS_UNCONSTRAINED is nonzero if the function returns an unconstrained
-   object.  RETURNS_BY_REF is nonzero if the function returns by reference. */
-extern tree create_subprog_type		PROTO((tree, tree, tree, int, int));
+   object.  RETURNS_BY_REF is nonzero if the function returns by reference. 
+   RETURNS_WITH_DSP is nonzero if the function is to return with a
+   depressed stack pointer.  */
+extern tree create_subprog_type		PROTO((tree, tree, tree, int, int,
+					       int));
 
 /* Return a copy of TYPE, in the same obstack as it was, but safe to modify
    in any way.  */
@@ -484,6 +487,9 @@ extern void process_attributes PROTO((tree, struct attrib *));
 /* Obtain any pending elaborations and clear the old list.  */
 extern tree get_pending_elaborations PROTO((void));
 
+/* Return nonzero if there are pending elaborations.  */
+extern int pending_elaborations_p PROTO((void));
+
 /* Save a copy of the current pending elaboration list and make a new
    one.  */
 extern void push_pending_elaborations PROTO((void));
