@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                          C Implementation File                           */
 /*                                                                          */
-/*                            $Revision: 1.172 $                            */
+/*                            $Revision: 1.181 $                            */
 /*                                                                          */
-/*          Copyright (C) 1992-1999, Free Software Foundation, Inc.         */
+/*          Copyright (C) 1992-2000, Free Software Foundation, Inc.         */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -639,16 +639,24 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
       while (TREE_CODE (right_operand) == UNCHECKED_CONVERT_EXPR
 	     && ((TREE_CODE (right_type) == RECORD_TYPE
 		  && ! TYPE_LEFT_JUSTIFIED_MODULAR_P (right_type)
-		  && ! TYPE_ALIGN_OK_P (right_type))
+		  && ! TYPE_ALIGN_OK_P (right_type)
+		  && ! TYPE_IS_FAT_POINTER_P (right_type))
 		 || TREE_CODE (right_type) == ARRAY_TYPE)
-	     && ((TREE_CODE (TREE_TYPE (TREE_OPERAND (right_operand, 0)))
-		  == RECORD_TYPE)
+	     && (((TREE_CODE (TREE_TYPE (TREE_OPERAND (right_operand, 0)))
+		   == RECORD_TYPE)
+		  && ! (TYPE_LEFT_JUSTIFIED_MODULAR_P
+			(TREE_TYPE (TREE_OPERAND (right_operand, 0))))
+		  && ! (TYPE_ALIGN_OK_P
+			(TREE_TYPE (TREE_OPERAND (right_operand, 0))))
+		  && ! (TYPE_IS_FAT_POINTER_P
+			(TREE_TYPE (TREE_OPERAND (right_operand, 0)))))
 		 || (TREE_CODE (TREE_TYPE (TREE_OPERAND (right_operand, 0)))
 		     == ARRAY_TYPE))
-	     && (right_type
-		 != find_common_type (right_type,
-				      TREE_TYPE (TREE_OPERAND
-						 (right_operand, 0)))))
+	     && (0 == (best_type
+		       == find_common_type (right_type,
+					    TREE_TYPE (TREE_OPERAND
+						       (right_operand, 0))))
+		 || right_type != best_type))
 	{
 	  right_operand = TREE_OPERAND (right_operand, 0);
 	  right_type = TREE_TYPE (right_operand);
@@ -680,7 +688,8 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
 	   || TREE_CODE (result) == IMAGPART_EXPR
 	   || TREE_CODE (result) == WITH_RECORD_EXPR
 	   || TREE_CODE (result) == UNCHECKED_CONVERT_EXPR
-	   || (TREE_CODE (result) == NOP_EXPR
+	   || ((TREE_CODE (result) == NOP_EXPR
+		|| TREE_CODE (result) == CONVERT_EXPR)
 	       && (TREE_CODE (TREE_TYPE (result))
 		   == TREE_CODE (TREE_TYPE (TREE_OPERAND (result, 0))))
 	       && (TYPE_MODE (TREE_TYPE (TREE_OPERAND (result, 0)))
@@ -702,7 +711,17 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
 	  && (! (TREE_CODE (TYPE_SIZE (operation_type)) != INTEGER_CST
 		 && contains_placeholder_p (TYPE_SIZE (operation_type)))))
 	{
-	  right_operand = convert (operation_type, right_operand);
+	  /* For a variable-size type, with both BLKmode, convert using
+	     CONVERT_EXPR instead of an unchecked conversion since we don't
+	     need to make a temporary (and can't anyway).  */
+	  if (TREE_CODE (TYPE_SIZE (operation_type)) != INTEGER_CST
+	      && TYPE_MODE (TREE_TYPE (right_operand)) == BLKmode
+	      && TREE_CODE (right_operand) != UNCONSTRAINED_ARRAY_REF)
+	    right_operand = build1 (CONVERT_EXPR, operation_type,
+				    right_operand);
+	  else
+	    right_operand = convert (operation_type, right_operand);
+
 	  right_type = operation_type;
 	}
 
@@ -735,6 +754,17 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
 	  || ! TREE_CONSTANT (TYPE_MIN_VALUE (right_type)))
 	mark_addressable (left_operand);
 
+      /* If the array is an UNCHECKED_CONVERT_EXPR from and to BLKmode
+	 types, convert it to a normal conversion since GCC can deal
+	 with any mis-alignment as part of the handling of compponent
+	 references.  */
+
+      if (TREE_CODE (left_operand) == UNCHECKED_CONVERT_EXPR
+	  && TYPE_MODE (TREE_TYPE (left_operand)) == BLKmode
+	  && TYPE_MODE (TREE_TYPE (TREE_OPERAND (left_operand, 0))) == BLKmode)
+	left_operand = build1 (CONVERT_EXPR, TREE_TYPE (left_operand),
+			       TREE_OPERAND (left_operand, 0));
+
       modulus = 0;
       break;
 
@@ -782,6 +812,20 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
 	  right_base_type = get_base_type (right_type);
 	}
 
+      /* If either object if an UNCHECKED_CONVERT_EXPR between two BLKmode
+	 objects, change it to a CONVERT_EXPR.  */
+      if (TREE_CODE (left_operand) == UNCHECKED_CONVERT_EXPR
+	  && TYPE_MODE (left_type) == BLKmode
+	  && TYPE_MODE (TREE_TYPE (TREE_OPERAND (left_operand, 0))) == BLKmode)
+	left_operand = build1 (CONVERT_EXPR, left_type,
+			       TREE_OPERAND (left_operand, 0));
+      if (TREE_CODE (right_operand) == UNCHECKED_CONVERT_EXPR
+	  && TYPE_MODE (right_type) == BLKmode
+	  && (TYPE_MODE (TREE_TYPE (TREE_OPERAND (right_operand, 0)))
+	      == BLKmode))
+	right_operand = build1 (CONVERT_EXPR, right_type,
+				TREE_OPERAND (right_operand, 0));
+
       /* If both objects are arrays, compare them specially.  */
       if ((TREE_CODE (left_type) == ARRAY_TYPE
 	   || (TREE_CODE (left_type) == INTEGER_TYPE
@@ -964,6 +1008,9 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
     = (TREE_CONSTANT (left_operand) & TREE_CONSTANT (right_operand)
        && op_code != ARRAY_REF);
 
+  if (op_code == ARRAY_REF && TYPE_VOLATILE (operation_type))
+    TREE_THIS_VOLATILE (result) = 1;
+
   /* If we are working with modular types, perform the MOD operation
      if something above hasn't eliminated the need for it.  */
   if (modulus != 0)
@@ -1080,13 +1127,16 @@ build_unary_op (op_code, result_type, operand)
 	       Likewise if we didn't find anything different for the inner
 	       operand or if OPERAND extracts direction from a padding type
 	       whose field has a self-referential size (to avoid infinite
-	       recursion below).  */
+	       recursion below) or if the size of the type or the offset
+	       are self-referential since we need an object in that case.  */
 	    if ((mode == VOIDmode && op_code != ATTR_ADDR_EXPR)
 		|| inner == operand
 		|| (TREE_CODE (op_type) == RECORD_TYPE
 		    && TYPE_IS_PADDING_P (op_type)
 		    && (contains_placeholder_p
-			(TYPE_SIZE (TREE_TYPE (TYPE_FIELDS (op_type)))))))
+			(TYPE_SIZE (TREE_TYPE (TYPE_FIELDS (op_type))))))
+		|| contains_placeholder_p (TYPE_SIZE (op_type))
+		|| (offset != 0 && contains_placeholder_p (offset)))
 	      goto common;
 
 	    /* If INNER is a padding type whose field has a self-referential
@@ -1119,6 +1169,22 @@ build_unary_op (op_code, result_type, operand)
 	  }
 	  break;
 
+	case CONSTRUCTOR:
+	  /* If this is just a constructor for a padded record, we can
+	     just take the address of the single field and convert it to
+	     a pointer to our type.  */
+	  if (TREE_CODE (type) == RECORD_TYPE && TYPE_IS_PADDING_P (type))
+	    {
+	      result
+		= build_unary_op (ADDR_EXPR, NULL_TREE,
+				  TREE_VALUE (CONSTRUCTOR_ELTS (operand)));
+	      result = convert (build_pointer_type (TREE_TYPE (operand)),
+				result);
+	      break;
+	    }
+
+	  goto common;
+
 	case NOP_EXPR:
 	  if (AGGREGATE_TYPE_P (type)
 	      && AGGREGATE_TYPE_P (TREE_TYPE (TREE_OPERAND (operand, 0))))
@@ -1533,13 +1599,23 @@ build_simple_component_ref (record_variable, component, field)
   if (field == 0)
     return 0;
 
+  /* If the record variable is an UNCHECKED_CONVERT_EXPR from and to BLKmode
+     types, convert it to a normal conversion since GCC can deal with any
+     mis-alignment as part of the handling of compponent references.  */
+  if (TREE_CODE (record_variable) == UNCHECKED_CONVERT_EXPR
+      && TYPE_MODE (TREE_TYPE (record_variable)) == BLKmode
+      && TYPE_MODE (TREE_TYPE (TREE_OPERAND (record_variable, 0))) == BLKmode)
+    record_variable = build1 (CONVERT_EXPR, TREE_TYPE (record_variable),
+			      TREE_OPERAND (record_variable, 0));
+
   /* It would be nice to call "fold" here, but that can lose a type
      we need to tag a PLACEHOLDER_EXPR with, so we can't do it.  */
   ref = build (COMPONENT_REF, TREE_TYPE (field), record_variable, field);
 
   if (TREE_READONLY (record_variable) || TREE_READONLY (field))
     TREE_READONLY (ref) = 1;
-  if (TREE_THIS_VOLATILE (record_variable) || TREE_THIS_VOLATILE (field))
+  if (TREE_THIS_VOLATILE (record_variable) || TREE_THIS_VOLATILE (field)
+      || TYPE_VOLATILE (record_type))
     TREE_THIS_VOLATILE (ref) = 1;
 
   return ref;
@@ -1576,7 +1652,9 @@ build_component_ref (record_variable, component, field)
 
    GNU_SIZE is the size of the object and ALIGN is the alignment.
    GNAT_PROC, if present is a procedure to call and GNAT_POOL is the
-   storage pool to use.  If not preset, malloc and free will be used.  */
+   storage pool to use.  If not preset, malloc and free will be used
+   except if GNAT_PROC is the "fake" value of -1, in which case we allocate
+   the object dynamically on the stack frame.  */
 
 tree
 build_call_alloc_dealloc (gnu_obj, gnu_size, align, gnat_proc, gnat_pool)
@@ -1671,6 +1749,8 @@ build_call_alloc_dealloc (gnu_obj, gnu_size, align, gnat_proc, gnat_pool)
 
   else if (gnu_obj)
     return build_call_1_expr (free_decl, gnu_obj);
+  else if (gnat_pool == -1)
+    return build (ALLOCATE_EXPR, ptr_void_type_node, gnu_size, gnu_align);
   else
     return build_call_1_expr (malloc_decl, gnu_size);
 }
@@ -1725,7 +1805,7 @@ build_allocator (type, init, result_type, gnat_proc, gnat_pool)
       storage = build_call_alloc_dealloc (NULL_TREE, size,
 					  TYPE_ALIGN (storage_type),
 					  gnat_proc, gnat_pool);
-      storage = convert (storage_ptr_type, storage);
+      storage = convert (storage_ptr_type, make_save_expr (storage));
 
       if (TREE_CODE (type) == RECORD_TYPE && TYPE_IS_PADDING_P (type))
 	{
