@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                          C Implementation File                           */
 /*                                                                          */
-/*                             $Revision: 1.60 $                            */
+/*                             $Revision: 1.61 $                            */
 /*                                                                          */
-/*          Copyright (C) 1992-1999 Free Software Foundation, Inc.          */
+/*          Copyright (C) 1992-2000 Free Software Foundation, Inc.          */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -203,8 +203,6 @@ main (int argc, char *argv [])
 #endif
   char *spawn_args [5];
   int  spawn_index = 0;
-  char *bind_src_file_name;
-  char *bind_obj_file_name;
   int   bind_file_name_len;
 
 #if defined (__EMX__) || defined(MSDOS)
@@ -334,15 +332,6 @@ main (int argc, char *argv [])
   retcode = portable_spawn (link_args);
   if (retcode != 0) exit (retcode);
 
-  /* Only keep the binder generated files if the debugging option was
-   * specified when the gnatbl was invoked since this implies that the user
-   * might want to break within the binder generated file.
-   */
-  if (!g_present) {
-    unlink (bind_src_file_name);
-    unlink (bind_obj_file_name);
-  }
-
 #ifdef MSDOS
   retcode = portable_spawn (coff2exe_args);
   if (retcode != 0) exit (retcode);
