@@ -6,7 +6,7 @@
 #-                                                                          --
 #-                             G N A T H T M L                              --
 #-                                                                          --
-#-                            $Revision: 1.22 $                             --
+#-                            $Revision: 1.29 $                             --
 #-                                                                          --
 #-          Copyright (C) 1998 Free Software Foundation, Inc.               --
 #-                                                                          --
@@ -46,7 +46,7 @@
 sub print_usage
 {
   print "Usage is:\n";
-  print "  $0 [switches] main_file[.adb]\n";
+  print "  $0 [switches] main_file[.adb] main_file2[.adb] ...\n";
   print "     -83       : Use Ada83 keywords only (default is Ada95)\n";
   print "     -cc color : Choose the color for comments\n";
   print "     -d        : Convert also the files which main_file depends on\n";
@@ -59,6 +59,8 @@ sub print_usage
   print "                 saved. Default is 'html/'\n";
   print "     -pfile    : Use file as a project file (.adp file)\n";
   print "     -sc color : Choose the color for symbol definitions\n";
+  print "     -Tfile    : Read the name of the files from file rather than the\n";
+  print "                 command line\n";
   print "This program attemps to generate an html file from an Ada file\n";
   exit;
 }
@@ -95,6 +97,20 @@ while ($_ = shift @ARGV)
     &print_usage if ($output_dir =~ /^-/ || $output_dir eq "");
   }
 
+  if (/^-T\s*(.*)$/)
+  {
+      my ($source_file) = ($1 eq "") ? shift @ARGV : $1;
+      local (*SOURCE);
+      open (SOURCE, "$source_file") || die "file not found: $source_file";
+      while (<SOURCE>) {
+	  @files = split;
+	  foreach (@files) {
+	      $_ .= ".adb" if (! /\.ad[bs]$/);	      
+	      push (@list_files, $_);
+	  }
+      }
+  }
+
   if (/^-cc\s*(.*)$/)
   {
       $comment_color = ($1 eq "") ? shift @ARGV : $1;
@@ -140,7 +156,7 @@ local (@obj_dir) = ();
 if ($standard_library) {
     open (PIPE, "gnatls -v | ");
     local ($mode) = "";
-    while (($_ = <PIPE>)) {
+    while (defined ($_ = <PIPE>)) {
 	chop;
 	s/^\s+//;
 	$_ = './' if (/<Current_Directory>/);
@@ -233,6 +249,7 @@ sub create_header
 sub protect_string
 {
     local ($string) = shift;
+    $string =~ s/&/&amp;/g;
     $string =~ s/</&lt;/g;
     $string =~ s/>/&gt;/g;
     return $string;
@@ -248,7 +265,7 @@ sub create_footer
   local ($adafile) = shift;
   local ($string) = "";
   $string = "</PRE>" if ($adafile ne "");
-  return $string . "</BODY></HTML>";
+  return $string . "</BODY></HTML>\n";
 }
 
 #########
@@ -449,6 +466,8 @@ sub find_file
 ##  Inserts a new reference in the list of references
 ##  Params: - Ref as it appears in the .ali file ($line$type$column)
 ##          - Current file for the reference
+##          - Current offset to be added from the line (handling of
+##            pragma Source_Reference)
 ##          - Current entity reference
 ##  Modifies: - %symbols_used
 ########
@@ -456,14 +475,16 @@ sub create_new_reference
 {
     local ($ref) = shift;
     local ($lastfile) = shift;
+    local ($offset) = shift;
     local ($currentref) = shift;
-    local ($refline, $type, $refcolumn);
+    local ($refline, $type, $refcol);
 
     ## Do not generate references to the standard library files if we
     ## do not generate the corresponding html files
     return if (! $standard_library && $lastfile =~ /$standard_file_regexp/);
     
     ($refline, $type, $refcol) = /(\d+)(.)(\d+)/;
+    $refline += $offset;
     if ($type eq "b" && $symbols {$currentref} eq 'f')
     {
 	local ($cref_file, $cref) = ($currentref =~ /([^\#]+).htm\#(.+)/);
@@ -472,7 +493,11 @@ sub create_new_reference
 	$symbols_used {"$lastfile#$refline\_$refcol"} = $currentref;
 	$symbols {"$lastfile.htm#$refline\_$refcol"} = "body";
     }
-    else
+
+    ## Do not generate cross-references for "e" and "t", since these point to the
+    ## semicolon that terminates the block -- irrelevant for gnathtml
+    
+    elsif ($type ne "e" && $type ne "t") 
     {
 	$symbols_used {"$lastfile#$refline\_$refcol"} = $currentref;
     }
@@ -485,10 +510,16 @@ sub create_new_reference
 sub parse_ali
 {
   local ($filename) = shift;
-  local ($currentfile) = "";
-  local ($currentref) = "";
-  local (@reffiles) = ();
+  local ($currentfile);
+  local ($currentref);
   local ($lastfile);
+
+  # Contains entries of the form  [ filename source_reference_offset]
+  # Offset needs to be added to the lines read in the cross-references, and are
+  # used when the source comes from a gnatchop-ed file. See lib-write.ads, lines
+  # with ^D in the ALI file.
+  local (@reffiles) = ();
+  
   
   open (ALI, &find_file ($filename)) || do {
     print "no ", &find_file ($filename), " file...\n";
@@ -502,25 +533,33 @@ sub parse_ali
   
   foreach (@ali)
   {
-    if (/^D\s+([\w\d.-]+)/)
+    if (/^D\s+([\w\d.-]+)\s+\S+ \S+( (\d+):(.*))?/)
     {
-      if ($dependencies)
-      {
-	push (@list_files, $1) unless (grep (/$1/, @list_files));
-      }
-      push (@reffiles, &http_string (&get_real_file_name ($1)));
+	# The offset will be added to each cross-reference line. If it is
+	# greater than 1, this means that we have a pragma Source_Reference,
+	# and this must not be counted in the xref information.
+	my ($file, $offset) = ($1, (defined $3) ? 2 - $3 : 0);
+
+	if ($dependencies)
+	{
+	    push (@list_files, $1) unless (grep (/$file/, @list_files));
+	}
+	push (@reffiles, [&http_string (&get_real_file_name ($file)), $offset]);
     }
-    elsif (/^X\s+\d+\s+([\w\d.-]+)/)
+    
+    elsif (/^X\s+(\d+)/)
     {
-      $currentfile = &http_string (&get_real_file_name ($1));
-      $lastfile = $currentfile;
+	$currentfile = $lastfile = $1 - 1;
     }
-    elsif (/^(\d+)(.)(\d+)[ *]([\w\d-.]|".")+\s+(.*)/ && $currentfile ne "")
+    
+    elsif (/^(\d+)(.)(\d+)[ *]([\w\d-.]|".")+\s+(.*)/ && defined $currentfile)
     {
-      next if (! $standard_library && $currentfile =~ /$standard_file_regexp/);
+      my ($line) = $1 + $reffiles[$currentfile][1];
+      next if (! $standard_library
+	       && $reffiles[$currentfile][0] =~ /$standard_file_regexp/);
       if ($xref_variable || $2 eq &uppercases ($2))
       {
-	$currentref = "$currentfile.htm#$1\_$3";
+	$currentref = $reffiles[$currentfile][0] . ".htm#$line\_$3";
 	$symbols {$currentref} = &to_type ($2);
 	$lastfile = $currentfile;
 	
@@ -528,8 +567,10 @@ sub parse_ali
 	
 	foreach (split (" ", $endofline))
 	{
-	  s/^(\d+)\|//   && do { $lastfile = $reffiles [$1 - 1]; };
-	  &create_new_reference ($_, $lastfile, $currentref);
+	    (s/^(\d+)\|//) && do { $lastfile = $1 - 1; };
+	    &create_new_reference
+		($_, $reffiles[$lastfile][0],
+		 $reffiles[$lastfile][1], $currentref);
 	}
       }
       else
@@ -537,13 +578,16 @@ sub parse_ali
 	$currentref = "";
       }
     }
-    elsif (/^\.\s(.*)/ && $currentfile ne "" && $currentref ne "")
+    elsif (/^\.\s(.*)/ && $reffiles[$currentfile][0] ne "" && $currentref ne "")
     {
-      next if (! $standard_library && $currentfile =~ /$standard_file_regexp/);
+      next if (! $standard_library
+	       && $reffiles[$currentfile][0] =~ /$standard_file_regexp/);
       foreach (split (" ", $1))
       {
-	s/^(\d+)\|//  && do { $lastfile = $reffiles [$1 - 1]; };
-	&create_new_reference ($_, $lastfile, $currentref);
+	  (s/^(\d+)\|//) && do { $lastfile = $1 - 1; };
+	  &create_new_reference
+	      ($_, $reffiles[$lastfile][0], $reffiles[$lastfile][1],
+	       $currentref);
       }
     }
   }
@@ -651,7 +695,6 @@ sub output_file
 		  }
 		  elsif ($symbols {"$filename.htm#$lineno\_$column"})
 		  {
-
 		      ##  A symbol can both have a link and be a reference for
 		      ##  another link, as is the case for bodies and
 		      ##  declarations
@@ -743,6 +786,7 @@ EOF
 </HTML>
 EOF
   ;
+  close (INDEX);
   
   open (MAIN, ">$output_dir/main.htm") || die "couldn't write $output_dir/main.htm";
   print MAIN &create_header (""),
@@ -798,7 +842,7 @@ sub create_file_index
       {
 	if ($last_letter ne '')
 	{
-	  print INDEX_FILE "</UL></BODY></HTML>";
+	  print INDEX_FILE "</UL></BODY></HTML>\n";
 	  close (INDEX_FILE);
 	}
 	$last_letter = &uppercases (substr ($_, 0, 1));
@@ -821,7 +865,7 @@ EOF
       ".htm TARGET=main>$_</A>\n";   ## Problem with TARGET when in no_frame mode!
     }
     
-    print INDEX_FILE "</UL></BODY></HTML>";
+    print INDEX_FILE "</UL></BODY></HTML>\n";
     close INDEX_FILE;
   }
   else
@@ -868,7 +912,7 @@ sub create_function_index
       {
 	if ($last_letter ne '')
 	{
-	  print INDEX_FILE "</UL></BODY></HTML>";
+	  print INDEX_FILE "</UL></BODY></HTML>\n";
 	  close (INDEX_FILE);
 	}
 	
@@ -905,7 +949,7 @@ EOF
       }
     }
     
-    print INDEX_FILE "</UL></BODY></HTML>";
+    print INDEX_FILE "</UL></BODY></HTML>\n";
     close INDEX_FILE;
   }
   else
