@@ -1128,22 +1128,41 @@ dbxout_type (type, full, show_arg_types)
 	  dbxout_type_index (type);
 	  fprintf (asmfile, ";0;127;");
 	}
-      else if (use_gnu_debug_info_extensions
-	       && (TYPE_PRECISION (type) > TYPE_PRECISION (integer_type_node)
-		   || TYPE_PRECISION (type) > HOST_BITS_PER_WIDE_INT))
-	{
-	  /* This used to say `r1' and we used to take care
-	     to make sure that `int' was type number 1.  */
-	  fprintf (asmfile, "r");
-	  dbxout_type_index (integer_type_node);
-	  fprintf (asmfile, ";");
-	  print_int_cst_octal (TYPE_MIN_VALUE (type));
-	  fprintf (asmfile, ";");
-	  print_int_cst_octal (TYPE_MAX_VALUE (type));
-	  fprintf (asmfile, ";");
-	}
-      else /* Output other integer types as subranges of `int'.  */
+
+      /* If this is a subtype of another integer type, always prefer to
+	 write it as a subtype.  */
+      else if (TREE_TYPE (type) != 0
+	       && TREE_CODE (TREE_TYPE (type)) == INTEGER_CST)
 	dbxout_range_type (type);
+	
+      else
+	{
+	  /* If the size is non-standard, say what it is if we can use
+	     GDB extensions.  */
+
+	  if (use_gnu_debug_info_extensions
+	      && TYPE_PRECISION (type) != TYPE_PRECISION (integer_type_node))
+	    fprintf (asmfile, "@s%d;", TYPE_PRECISION (type));
+
+	  /* If we can use GDB extensions and the size is wider than
+	     an integer or we may have trouble writing the bounds the
+	     usual way, write them in octal.  */
+	  if (use_gnu_debug_info_extensions
+	      && (TYPE_PRECISION (type) > TYPE_PRECISION (integer_type_node)
+		  || TYPE_PRECISION (type) > HOST_BITS_PER_WIDE_INT))
+	    {
+	      fprintf (asmfile, "r");
+	      dbxout_type_index (type);
+	      fprintf (asmfile, ";");
+	      print_int_cst_octal (TYPE_MIN_VALUE (type));
+	      fprintf (asmfile, ";");
+	      print_int_cst_octal (TYPE_MAX_VALUE (type));
+	      fprintf (asmfile, ";");
+	    }
+	  else /* Output other integer types as subranges of `int'.  */
+	    dbxout_range_type (type);
+	}
+
       CHARS (22);
       break;
 
